#include <QtGui/QApplication>
#include <popt.h>
#include "calibration.h"

int main( int argc, char *argv[] ) {

  // set variables to default
  int size_x = SIZE_X,
    size_y = SIZE_Y,
    size_window = WINDOW,
    preview = PREVIEW;
  
  double min_corr = MIN_CORR;

  char *device = VIDEO_DEVICE;

  // popt options
  struct poptOption optionsTable[] = {

    { "device", 'd', POPT_ARG_STRING | POPT_ARGFLAG_SHOW_DEFAULT,
      &device, 0, "Specify video4linux device", "device-name" },

    { "width", 'w', POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT,
      &size_x, 0, "Width of the chessboard (number of black squares)" ,
      "width" },

    { "height", 'h', POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT,
      &size_y, 0, "Height of the chessboard (number of black squares)",
      "height"},

    { "window", '\0', POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT,
      &size_window, 0, "Size of the window for corner detection",
      "size"},

    { "correlation", '\0', POPT_ARG_DOUBLE | POPT_ARGFLAG_SHOW_DEFAULT,
      &min_corr, 0, "Specify minimum correlation to be calculated"
      "to assume that the corners are accurately found", "corr" },

    { "preview", 'p', POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT,
      &preview, 0, "Show the preview window", "0/1"},

    POPT_AUTOHELP

    POPT_TABLEEND 

  };

  poptContext optCon( poptGetContext( NULL, argc, (const char **)argv,
                                      optionsTable, 0 ) );

  try {

    // Parse command-line.
    char c;
    while ( ( c = poptGetNextOpt( optCon ) ) >= 0 );

    const char *strayParameter = poptGetArg( optCon );
    MMERROR( strayParameter == NULL, mimasexception, ,
             "Unexpected command-line argument \"" << strayParameter
             << "\"" );

    // Throw exception on error.
    MMERROR( c >= -1, mimasexception, ,
             poptBadOption( optCon, POPT_BADOPTION_NOALIAS ) << ": "
             << poptStrerror( c ) );

    QApplication app( argc, argv );

    Calibration h(size_x, size_y, size_window, min_corr, device, preview);
    h.exec();

    return 0;

  } catch ( std::exception &e ) {

    cerr << e.what() << endl;
    poptPrintUsage( optCon, stderr, 0 );
    return 1;

  };
}
