//
// Example usage of L-systems
// Bala Amavasai (bala@amavasai.org)
//
// Mon Dec 19 17:17:26 GMT 2005
//
//
#include <iostream>
#include <fstream>
#include <boost/algorithm/string.hpp>
#include <string>
#include <list>
#include "image.h"
#include "image_mesaoutput.h"
#include "rgba.h"
#include "stdlib.h"
#include "lsystem.h"

using namespace std;
using namespace mimas;



int	main(int argc, char **argv)
{

	int retVal = 0;
	try {
		x11_display display;
		image< rgba<unsigned char> > image;
		image_mesaoutput< rgba<unsigned char> > window( &display );

		image.init(800,800);
		image.fill(rgba<unsigned char> (255,255,255,255));
		lsystem lsys;

		// Display the inbuilt demonstration L-system
		lsys.iterate(5);
		lsys.drawLsystem(image, 400, 700, rgba<unsigned char> (0, 127, 0, 0) );
		window << image << mimas::pause();

		// Set up a new L-system
		// 7th Order Terdragon
		cout << "Displaying the 7th order Terdragon" << endl;
		lsys.setLstring("F");
		lsys.setAxiom("F");
		lsys.setRule("F+F-F");
		lsys.setAngleStep(120);
		lsys.iterate(7);
		lsys.setLineLength(10);
		image.fill(rgba<unsigned char> (255,255,255,255)); // blank the canvas
		lsys.drawLsystem(image, 600, 200, rgba<unsigned char> (127, 127, 0, 0) );
		window << image << mimas::pause();





	} catch ( exception &e ) {
		cerr << e.what() << endl;
		retVal = 1;
	};
	return retVal;
}


