#ifndef MMVL_COMPENSATE_SPEED_HH
#define MMVL_COMPENSATE_SPEED_HH

#include <boost/numeric/ublas/vector.hpp>

//  !!Make it (more?) generic relative to the hypothesis
//The functionnality of this class depends on the previous result
//that are given by the analyse_result policy and should change the
//observation made by the observer.
//a default class that implement nothing should be provided too.

#include "hypothesis2D.h"
#include "hypothesis_filter.h"

namespace mimas{
  namespace hf{

    /**A very basic implementation of dynamic adaptation of the filter.
       It takes into account the speed on the two previous frames.
       (More sophisticated things might be intended such as taking
       into account the maximum speed, the average speed, the direction,
       the acceleration...)

       If the position of the object has been able to be evaluated on the
       2 previous frames the infered speed is used to outweigh the hypothesis
       that are located within a similar distance. 
       Note that the direction of the movement is not taken into account.
       For a not purely stochastic movements (say linear) I imagine this would
       improve most the compensation.
    */
    
    template<typename Hypothesis_T>    
    class compensate_speed
    { 
    public:
      //!!!As this class is working it needs 2D hypothesis at the moment 
      //the type enforce the need (can be used as a generic technic)
      //!!!!!!!should be easy to improve it for 3D implementing
      //a distance between 2 hypothesis and changing Point to Hypothesis_T
      inline void save_location(const hypothesis2D &h)
      {
	//We are taking the assumption that if AnalyseResult doesn't
	//give result it gives nan for the location however
	//!!!(I think it's not a good assumption as it make
	//forces a supposed independant class to take more responsabilities
	
	Point p(2);
	p(0) = h.x;
	p(1) = h.y;	

	//std::cerr<< "Location is: "<< p(0) << "; "<< p(1) <<std::endl;

	if (probable_location_instanciated)
	  {
	    previous_location = probable_location;
	    previous_location_instanciated = true;
	  }
	else
	  {
	    previous_location_instanciated = false;
	  }

	//!!this is not very robust find another way
	//to detect a "tracking failure"
	if (!isnan(p(0)) && !isnan(p(1)))
	  {
	    probable_location = p;
	    probable_location_instanciated = true;
	  }
	else
	  probable_location_instanciated = false;
      }

      compensate_speed()
	:previous_location(2), probable_location(2), velocity(2), 
	 previous_location_instanciated(false), probable_location_instanciated(false) 
      {}
      
      void set_compensate(float to_determine = .05, float factor = 2.)
      {
	FACTOR = factor;
	TO_DETERMINE = to_determine;
      }
   
    protected:
      void outweigh(Hypothesis_T &h);

    private:
      Point previous_location;
      Point probable_location;
      Point velocity;
      
      bool previous_location_instanciated;
      bool probable_location_instanciated;
      
      float FACTOR;
      float TO_DETERMINE;
      float abs_val(float i){ return i>0 ? i: 0 - i;} 
    };

#include "compensate_speed.tcc"
  }
}
#endif
