/**********************************************************************
*example code how to use the hypothesis filter.
*
*
* @author: Manuel Boissenin
* @date: 20/05/2004
*
* Time-stamp: <2005-06-02 18:37:58 engmb>
* Copyright (C) 2004 by Manuel Boissenin
*
 ***********************************************************************/
#include <string>
#include <fstream>
#include <iostream>
#include <iomanip>
#include <sstream>

//#include <x11_display.h>
#include <segment_hashmap_tool.h>
#include <mesa_offscreen.h>
#include <image_mesaoutput.h>
#include <binmorph_tool.h>

#include <image_funcs.h>
#include <property_image.h>
#include <edge_string_op.h>
#include <canny_tool.h>

#include <image_fileoutput.h>
#include <image.h>
#include <image_fileinput.h>
#include <image_mesaoutput.h>
#include <hypothesis3D.h>
#include <image_loader.h>
#include <rgba.h>
#include <p_condensation.h>
#include <analyse_result_blob.h>

#include "openGL_observer.h"
#include "find_line.h"

using namespace mimas::hf;

int main(int argc, char** argv)
{
  int retVal = 0;
  try {

    //Initialisation tracker
    x11_display *dpy =new x11_display;

    //glutInit(&argc, argv);
    hypothesis3D init(atof(argv[2]),atof(argv[3]),atof(argv[4]),atof(argv[5]),atof(argv[6]),atof(argv[7]));
  
    hypothesis3D::transX = atof(argv[8]);
    hypothesis3D::transY = atof(argv[9]);
    hypothesis3D::transZ = atof(argv[10]);
    hypothesis3D::rotX = atof(argv[11]);
    hypothesis3D::rotY = atof(argv[12]);
    hypothesis3D::rotZ = atof(argv[13]);

    image_loader<rgba<unsigned char> >::set il_param(argv[1],99,".ppm",2);;
    openGL_observer<hypothesis3D, rgba<unsigned char> >::set obs_param(dpy);
    analyse_result_blob<hypothesis3D, rgba<unsigned char> >::set analyse_param(.35, 60, 45);
  
    hypothesis_filter<image_loader, openGL_observer, analyse_result_blob, hypothesis3D,
      p_condensation, no_compensation, mimas::rgba<unsigned char> > hf(init, atoi(argv[14]), il_param, obs_param, analyse_param);

    //tracking
    //    image_mesaoutput<rgba<unsigned char> > show(dpy, hf.image.getWidth(), hf.image.getHeight());

    //!!!load the distance image, better to calculate it live for a real-time demonstration

    //////std::string distance_image_basename = argv[1];
    //////distance_image_basename += "dist";
    
    ///////image_loader<rgba<unsigned char> > distance_map
    ///////(hf.filteredImage,
    ///////image_loader<rgba<unsigned char> >::set(distance_image_basename, 99, ".ppm", 2, 0));

    std::string directory = argv[15];
    
    std::string filteredBaseName = directory + "/Filtered";
    std::string fileOutputBaseName = directory + "/Result";
    int imageNumber = 0;
    
    while(true)
      {	
	try{hf.next();
	}
	catch (mimasexception &e){
	  std::cerr<<e.what()<<std::endl;  
	  return 0;
	}


	//---------------color segmentation of the image -----------------//
	segment_hashmap_tool seg;

	// load in the images that contain sample colours
	image< rgba< unsigned char > > green;
	std::ifstream imFile( "seq1/green.ppm", std::ios::binary ); imFile >> green;

	image< rgba< unsigned char > > green2;
	std::ifstream imFile6( "seq1/green2.ppm", std::ios::binary ); imFile6 >> green2;

	
	//	image< rgba< unsigned char > > white;
	//	std::ifstream imFile2( "seq1/white.ppm", std::ios::binary ); imFile2 >> white;
	
	//	image< rgba< unsigned char > > white1;
	//	std::ifstream imFile3( "seq1/white1.ppm", std::ios::binary ); imFile3 >> white1;
	
	//	image< rgba< unsigned char > > orange;
	//	std::ifstream imFile4( "seq1/orange.ppm", std::ios::binary ); imFile4 >> orange;
	
	//	image< rgba< unsigned char > > orange1;
	//	std::ifstream imFile5( "seq1/orange1.ppm", std::ios::binary ); imFile5 >> orange1;
	

	// store the colours from the first image

	seg.storeColours(green);
	seg.storeColours(green2);

	//seg.storeColours(white);
	//seg.storeColours(white1);
	//seg.storeColours(orange);
	//seg.storeColours(orange1);
	

	image< rgba< unsigned char > > simout;
	seg.segment(hf.image, simout, 40);

	//------------------spread the segmented region----------------------//
	
	image<int> segmented = image<int>(simout);
	binmorph_tool morph_tool;

	image<int> segmented2;
	morph_tool.greyToBinary(segmented, segmented2 , 50);

	morph_tool.setSizeStructElement(35);
	morph_tool.dilation(segmented2, segmented);
	
	//----------------------find the edges of the image ----------------//
	
	image<int> input = image<int>(hf.image);
	canny_tool<int> canny;
	edge_string_ptr_list	edge_strings;
	edgestring_image eim;
	
	edge_string_op::useFastMode();
	edge_strings=canny.findEdgeStrings( input );

	//(2) then create an image to contain them for visualisation
	// eim.setOwnership(false);
	eim.init( input.getWidth(), input.getHeight());
	eim.addStrings( edge_strings );


	image< rgba< unsigned char > > temp;
	temp.init( eim.getWidth(), eim.getHeight() );
	
	int i,j;
	for(j=0;j<temp.getHeight();j++)
	  {
	    for(i=0;i<temp.getWidth();i++)
	      {
		prop_ptr p( eim.pixel(i,j) );
		
		if( p )
		  {
		    p->draw( temp, i, j, eim.getDrawContrast() );
		  }
	      }
	  }

	image<int> tempEdge = image<int>(temp);
	
	
	//----------------------and the two images ----------//
	
	image<int> anded_image = morph_tool.image_and(tempEdge, segmented);


	//---------------------calculate the distance map image--------------//

	//binmorph_tool tools;
	//tools.distanceTransform(anded_image ,hf.filteredImage); 
	
	//Save the result of the filtering of the image.

	std::ostringstream str2;
	str2 << filteredBaseName << std::setw(3) << std::setfill( '0' ) <<  imageNumber++
	    << ".ppm";
	std::ofstream filteredOut(str2.str().c_str(), std::ios::binary); 

	//filteredOut << temp;
	//filteredOut << simout;
	//filteredOut << segmented;
	//filteredOut << anded_image;
	//filteredOut << hf.filteredImage;


	//image<unsigned char> edgeImage = image<unsigned char>(temp);

	find_line<image<int >::iterator > h(anded_image.ul(), anded_image.lr() , .45, 140);    
	//find_line<image<unsigned char>::iterator > h(edgeImage.ul(), edgeImage.lr() , 5 , 180);    
    
	filteredOut<< anded_image;
	filteredOut<< h.exec();
	/*
	std::ostringstream str;
	str << fileOutputBaseName << std::setw(3) << std::setfill( '0' ) <<  imageNumber++
	    << ".ppm";
	std::ofstream fileOut(str.str().c_str(), std::ios::binary); 

	std::vector<hypothesis3D> estimatedLocation;
	//filter the image and instanciate it for the observer

	//load the distance map to edges of the image.

	//hf.filteredImage = gradient(hf.image.ul(), hf.image.lr(),40);
	
	//show << hf.image<< mimas::delay(500);
	//show << hf.filteredImage<< mimas::delay(500);
	hf.filter();
	//show << hf.offscreenImage<<mimas::delay(500);
	
	estimatedLocation = hf.analyse();

	rgba<unsigned char> col(0,255,255);
	hf.draw(estimatedLocation, col);     //from Observer	

	//find the best peak and do a partial reinitialisation around it
	std::vector<hypothesis3D>::iterator it_h, it_h_max;
	float temp_weight = 0;
	
	for(it_h = estimatedLocation.begin(); it_h != estimatedLocation.end(); it_h++)
	  if (it_h->weight > temp_weight)
	    {
	      temp_weight = it_h->weight;
	      it_h_max = it_h;
	    }
	///if an hypothesis has an excellent result we did a partial_reset around it

	for(it_h = hf.hypotheses.begin(); it_h != hf.hypotheses.end(); it_h++)
	  if (it_h->weight > .85)
	    {
	      hf.partial_reset(*it_h, .05, 0, .1, true);
	      std::cerr<<"One partial reinitialisation done" <<std::endl;
	    }

	
	//Draw the information on the image
	if(temp_weight != 0)
	  {
	    rgba<unsigned char> red(255, 0, 0);
	    //hf.partial_reset(*it_h_max, .04, .2);
	    std::cerr<< "Max weight is: "<< temp_weight << std::endl;
	    std::cerr<< "For hypothesis: "<< std::endl;
	    hf.draw(*it_h_max, red);

	    ///Write result on the screen

	    mesa_offscreen m(dpy, hf.image, false);
	    m.ortho2D();
	    m.setColor(red);
	    
	    glPushMatrix();
	    glTranslated(10, 15, 0 );
	    glScaled( .1, -.1, 1.0 );
	    //glRotatef( angle, 0, 0, 1 );
	    std::ostringstream str2;
	    str2 << "X: "<< (*it_h_max).x;
	    m.text(str2.str());
	    glPopMatrix();
  
  
	    glPushMatrix();
	    glTranslated(10, 30, 0 );
	    glScaled( .1, -.1, 1.0 );
	    //glRotatef( angle, 0, 0, 1 );
	    std::ostringstream str3;
	    str3  << "Y: " << (*it_h_max).y ;
	    m.text(str3.str());
	    glPopMatrix();
	    
	    
	    glPushMatrix();
	    glTranslated(10, 45, 0 );
	    glScaled( .1, -.1, 1.0 );
	    //glRotatef( angle, 0, 0, 1 );
	    std::ostringstream str4;
	    str4 <<  "Z: " <<  (*it_h_max).z; 
	    m.text(str4.str());
	    glPopMatrix();

	    
	    glPushMatrix();
	    glTranslated(10, 60, 0 );
	    glScaled( .1, -.1, 1.0 );
	    //glRotatef( angle, 0, 0, 1 );
	    std::ostringstream str5;
	    str5 << "Rx: "<< (*it_h_max).Rx;
	    m.text(str5.str());
	    glPopMatrix();

  
	    glPushMatrix();
	    glTranslated(10, 75, 0 );
	    glScaled( .1, -.1, 1.0 );
	    //glRotatef( angle, 0, 0, 1 );
  	    std::ostringstream str6;
	    str6 <<"Ry: "<< (*it_h_max).Ry ;
  	    m.text(str6.str());
	    glPopMatrix();

  
	    glPushMatrix();
	    glTranslated(10, 90, 0 );
	    glScaled( .1, -.1, 1.0 );
	    //glRotatef( angle, 0, 0, 1 );
    	    std::ostringstream str7;
	    str7 << "Rz: "<< (*it_h_max).Rz;
  	    m.text(str7.str());
	    glPopMatrix();
  
	    m.finish();
	    
	  }
	
	fileOut << hf.image;
	*/
      }

  } catch ( std::exception &e ) {
    std::cerr << e.what() << std::endl;
    retVal = 1;
  };
  return retVal;
}





