#ifndef MMVL_MAPPING
#define MMVL_MAPPING

#include <colour_properties_tool.h>
#include <iostream>
#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/io.hpp>

#include <sys/timeb.h>
#include <cassert>
#include <vector>

/**************************************************************
 *Manuel Boissenin
 *march 26 2004
 *
 *
 *http://www.vision.caltech.edu/bouguetj/calib_doc/
 **************************************************************/

using std::cerr;
using std::cout;
using std::endl;
using std::vector;
//using namespace mimas;

typedef boost::numeric::ublas::vector<double> Point;
typedef boost::numeric::ublas::vector<double> vec;
typedef boost::numeric::ublas::matrix<double> mat;

namespace mimas{

  class figure
    {
    public:
      virtual void getPoints(vector<Point> &points, int number) =0;
      virtual ~figure(){}
    };

  class linesegment:public figure
    {
    public:
      Point a;
      Point b;
  
      linesegment(Point &a, Point &b)
	:a(a), b(b){}
  
      void getPoints(vector<Point> &points, int n)
	{
	  for(double p= 0; p <= n; p++)
	    {
	      points.push_back(a*(1 - p/double(n)) + b*(p/double(n)));
	    }
	}
    };

  class parallelepiped:public figure
    {
    public:
      mat Rotation;

      Point A, B, C, D, E, F, G, H;

      vector<Point> initial_points;
      vector<Point>::iterator itpi;

      vector<Point> moved_points;
      vector<Point>::iterator itpm;
  
      vector<linesegment> segments;
      vector<linesegment>::iterator it;
      // AB, AD, AE, CB, CD, CG, HD, HE, HG, FB, FE, FG;

      parallelepiped(double width, double height, double depth)
	:Rotation(3,3), A(3) , B(3), C(3), D(3), E(3), F(3), G(3), H(3)
	{
	  A(0) = -width/2;
	  A(1) = -height/2;
	  A(2) = -depth/2;

	  B(0) = width/2;
	  B(1) = -height/2;
	  B(2) = -depth/2;

	  C(0) = width/2;
	  C(1) = height/2;
	  C(2) = -depth/2;

	  D(0) = -width/2;
	  D(1) = height/2;
	  D(2) = -depth/2;

	  E(0) = -width/2;
	  E(1) = -height/2;
	  E(2) = depth/2;

	  F(0) = width/2;
	  F(1) = -height/2;
	  F(2) = depth/2;

	  G(0) = width/2;
	  G(1) = height/2;
	  G(2) = depth/2;

	  H(0) = -width/2;
	  H(1) = height/2;
	  H(2) = depth/2;
    
	  initial_points.push_back(A);
	  initial_points.push_back(B);
	  initial_points.push_back(C);
	  initial_points.push_back(D);
	  initial_points.push_back(E);
	  initial_points.push_back(F);
	  initial_points.push_back(G);
	  initial_points.push_back(H);
	}

      //Euler rotation matrix include it in mimas library famous matrix 
      void rotate(vec R)//!
	{
	  moved_points.clear();

	  //!!
	  double pitch = R(0);
	  double yaw = R(1);
	  double roll = R(2);
	  Rotation(0,0) = cos(yaw) * cos(roll);
	  Rotation(0,1) = sin(pitch)*sin(yaw)*cos(roll) + cos(pitch)*sin(roll);
	  Rotation(0,2) = sin(pitch)*sin(roll)-cos(pitch)*sin(yaw)*cos(roll);
	  Rotation(1,0) = (0-cos(yaw)*sin(roll));
	  Rotation(1,1) = cos(pitch)*cos(roll) - sin(pitch)*sin(yaw)*sin(roll);
	  Rotation(1,2) = cos(pitch)*sin(yaw)*sin(roll) + sin(pitch)*cos(roll);
	  Rotation(2,0) = sin(yaw);
	  Rotation(2,1) = 0 - sin(pitch)*cos(yaw);
	  Rotation(2,2) = cos(pitch)*cos(yaw);
     
	  for(itpi = initial_points.begin(); itpi != initial_points.end(); itpi++)
	    {	
	      moved_points.push_back(prod(Rotation, *itpi)); //!! here we need to create a point
	    }
	}

      /// Has to be used after points had been rotated
      void translate(Point &T)
	{
	  for(itpm = moved_points.begin(); itpm != moved_points.end(); itpm++)
	    {
	      *itpm += T;
	    }
	}

      void getPoints(vector<Point> &oldpoints,int n) 
	{
	  segments.clear();
    
	  segments.push_back(linesegment (moved_points[0], moved_points[1]));
	  segments.push_back(linesegment (moved_points[0], moved_points[3]));
	  segments.push_back(linesegment (moved_points[0], moved_points[4]));
	  segments.push_back(linesegment (moved_points[2], moved_points[1]));
	  segments.push_back(linesegment (moved_points[2], moved_points[3]));
	  segments.push_back(linesegment (moved_points[2], moved_points[6]));
	  segments.push_back(linesegment (moved_points[7], moved_points[3]));
	  segments.push_back(linesegment (moved_points[7], moved_points[4]));
	  segments.push_back(linesegment (moved_points[7], moved_points[6]));
	  segments.push_back(linesegment (moved_points[5], moved_points[1]));
	  segments.push_back(linesegment (moved_points[5], moved_points[4]));
	  segments.push_back(linesegment (moved_points[5], moved_points[6]));     
	  for(it = segments.begin(); it != segments.end(); it++)
	    {
	      it->getPoints(oldpoints, n);
	    } 
	}
    };

  class mapper{
  private:
    //intrisic parameters 
    mat cameraCalibration;
    vec kc;
  
    vector<figure *> figures;
    vector<figure *>::iterator itf;
  
    vector<Point> setPoint;
    vector<Point>::iterator it;

    vector<Point> mappedPoint;
    vector<Point>::iterator itm;

    //factor*Xn doesn't work but Xn*factor works
    //make this code easier/shorter/beautifull finding a way to fill a matrix more easily
    Point map(Point &P)
      {
	MMERROR( P(2) > 0, mimasexception, ,
		 "mapper:= in functiun rewrite_point Z value must be different from 0" );

	Point Xn(2);
	Point dx(2);
	Point Xd(2);    
	Point XdBis(3);    
	Point Xp(3);

	Xn(0)= P(0)/P(2);
	Xn(1)= P(1)/P(2);
    
	double r2 = Xn(0)*Xn(0) + Xn(1)*Xn(1);
	double factor = 1 + kc(0)*r2 + kc(1)*r2*r2;

	dx(0) = 2*kc(2)*Xn(0)*Xn(1) + kc(3)*(r2 + 2*Xn(0)*Xn(0));
	dx(1)= kc(2)*(r2 + 2*Xn(1)*Xn(1)) + 2*kc(3)*Xn(0)*Xn(1); 
    
	Xd = Xn * factor + dx;

	XdBis(0) = Xd(0);   
	XdBis(1) = Xd(1);
	XdBis(2) = 1;
  
	Xp = prod(cameraCalibration, XdBis);

	return Xp;
      }

  public:

    void add_figure(figure &fig)
      {
	cerr<<"adding figure" <<endl;
	figures.push_back(&fig);
      }
  
    void getPoints(int n)
      {
	setPoint.clear();
	for(itf = figures.begin(); itf != figures.end(); itf++)
	  {
	    (**itf).getPoints(setPoint,n);
	  }
      }

    void mapPoints(void)
      {
	//cerr<<"mapping points" <<endl;
	mappedPoint.clear();
	for(it = setPoint.begin(); it != setPoint.end(); it++)
	  {
	    //  cerr<<"Position is: "<< "X: " <<(*it) (0) <<" Y: "<< (*it) (1)<< " Z: " << (*it) (2)<<endl;
	    mappedPoint.push_back(map(*it));
	  }
      }

    int approx(double n)
      {
	double p = int(n);
	if (n - p < .5)
	  return int(p);
	return int(p+1);
      }

    template <typename Pixel_T> 
      void plot(image<Pixel_T> &image, Pixel_T color = Pixel_T(255))
      {
	//cerr<<"plotting points" <<endl;
	for(itm = mappedPoint.begin(); itm != mappedPoint.end(); itm++)
	  {
	    //cerr<<"Position is: "<< "X: " <<(*itm) (0) <<"Y: "<< (*itm) (1)<<endl;
	    image.setPixel(approx((*itm)(0)) ,approx((*itm)(1)), color);
	  }
      }
  
    mapper(float fc1,float fc2, float cc1, float cc2,float kc1,float kc2, float kc3, float kc4):
      cameraCalibration(3,3) , kc(4)
      {

	cameraCalibration(0,0) = fc1;
	cameraCalibration(0,2) = cc1;
	cameraCalibration(1,1) = fc2;
	cameraCalibration(1,2) = cc2;
	cameraCalibration(2,2) = 1;
	cout<< "camera calibration matrix:"<<cameraCalibration <<endl;
     
	kc(0) = kc1;
	kc(1) = kc2;
	kc(2) = kc3;
	kc(3) = kc4;
    
	cout<< "kc parameters:"<<kc <<endl;
      }
  };
}
#endif
