template<typename Hypothesis_, typename Pixel_>
void openGL_observer<Hypothesis_, Pixel_>::observe(hypothesis3D &h)
{
  double dist = calculateDistance(h);

  //here a gradient descent is done if the hypothesis is already good   
  if (dist > .1)
    {
      dist = gradientDescent(h, dist);
    }

#ifndef NDEBUG
  if(dist > .5)		
    h.debug();
#endif
  
  //  if(dist> 0)
    std::cerr<<"value is:"<<dist<<std::endl;
  
  float alpha = 255;
  if (dist > 1) ;
  else if(dist >0) alpha *= dist;
  else alpha = 0;
  
  //Draw rubik on image using transparency 
  for(std::vector<Point>::iterator itm = pixels.begin(); itm != pixels.end(); itm++)
      {
	image_ref.getPixel((int)(*itm)(0) , (int)(*itm)(1)).overlay(255, 255, 255, (unsigned char)(alpha));
      }
  
  if (dist > 0)
    h.weight = dist;
  else
    h.weight = 0;          
}

template<typename Hypothesis_T, typename Pixel_T>
double openGL_observer<Hypothesis_T, Pixel_T>::calculateDistance(hypothesis3D &h)
{
  pixels.clear();

  //draw with OpenGL the rubik's contours and take the coordinate
  //of the visible points.

  //std::cerr<< "position: "<< h.x <<","<< h.y <<","<< h.z <<","<<std::endl;

  offscr = new mesa_offscreen<Pixel_T>(disp, offscreenImage, true);
  drawCube::projection(offscreenImage.getWidth(), offscreenImage.getHeight());
  drawCube::init();
  drawCube::positionRubik(h.x,h.y,h.z,h.Rx,h.Ry,h.Rz);
  offscr->finish();

  //
  //#ifndef NDEBUG
  //for debuging purposes, display an image of the hypothesis
  image_mesaoutput<rgba<unsigned char> > show(disp);
  show << offscreenImage;
  //#endif
  
  
  //offscreenImage.setPixel(10, 10, Pixel_T(10));
  
  for(int y = 0; y < offscreenImage.getHeight(); y++)
    for(int x=0; x < offscreenImage.getWidth(); x++)
	if(offscreenImage.getPixel(x,y) != Pixel_T())
	  {
	    Point tmp_pt(2);
	    tmp_pt(0) = x;
	    tmp_pt(1) = offscreenImage.getHeight() - y;
	    pixels.push_back(tmp_pt);   
	  }
  
  //!!! I don't see where the offscreen image is reset to 0

 //Take measurements on image data and display it (for testing purpose)
  std::cerr<< "a2"<<std::endl;
  return evaluateDistance();
}

template<typename Hypothesis_T, typename Pixel_T>
double openGL_observer<Hypothesis_T, Pixel_T>::evaluateDistance(void)
{
  double N=0;


  std::cerr<<"Number of points: "<< pixels.size() <<std::endl;
  for(std::vector<Point>::iterator itm = pixels.begin(); itm != pixels.end(); itm++)
      {
	N += 2.3 - int(filteredImage.getPixel((int)(*itm)(0) , (int)(*itm)(1)));
	//N +=  int(filteredImage.getPixel((int)(*itm)(0) , (int)(*itm)(1)));
      }

  N /= pixels.size();
  
  return N;
}

template<typename Hypothesis_T, typename Pixel_T>
float openGL_observer<Hypothesis_T, Pixel_T>::gradientDescent(hypothesis3D &h, double &dist)
{
  cerr<<"entering gradientDescent()"<<endl;
  hypothesis3D test = h;
  bool improved = false;
  double temp; 

  //We can implement introspection facilities to make generic this process (linked with the hypothesis class)
  test.x++;
  temp = calculateDistance(test);
  double x_ = temp - dist ; //derivation along this variable
  test = h; //reinitialisation
  
  test.y++;
  temp = calculateDistance(test);
  double y_ = temp - dist ; //derivation along this variable
  test = h; //reinitialisation
  
  test.z++;
  temp = calculateDistance(test);
  double z_ = temp - dist ; //derivation along this variable
  test = h; //reinitialisation

  
  test.Rx++;
  temp = calculateDistance(test);
  double Rx_ = temp - dist ; //derivation along this variable
  test = h; //reinitialisation
  
  test.Ry++;
  temp = calculateDistance(test);
  double Ry_ = temp - dist ; //derivation along this variable
  test = h; //reinitialisation
  
  test.Rz++;
  temp = calculateDistance(test);
  double Rz_ = temp - dist ; //derivation along this variable
  test = h; //reinitialisation

  double norm = sqrt(x_*x_ + y_*y_ + z_*z_ + Rx_*Rx_ +Ry_*Ry_ + Rz_*Rz_) ;

  //Now move along the gradient
  test = h;
  double DISTANCE_PARAM = 1.6;
  
  test.x += (x_ / norm) * DISTANCE_PARAM;
  test.y += (y_ / norm) * DISTANCE_PARAM;
  test.z += (z_ / norm) * DISTANCE_PARAM;
  test.Rx += (Rx_ / norm) * DISTANCE_PARAM;
  test.Ry += (Ry_ / norm) * DISTANCE_PARAM;
  test.Rz += (Rz_ / norm) * DISTANCE_PARAM;

  temp = calculateDistance(test);
 
  if(temp > dist)
    {
      improved = true;
      std::cerr<<"value was: "<< dist<<std::endl;
      std::cerr<<"Now is: "<< temp<<std::endl;
      std::cerr<< "Improved by:"<< temp - dist<<std::endl;

      //Save/Use the improved values
      dist = temp;
      h = test;
      //and 1 more time
      dist = gradientDescent(h, dist);
    }
  else{
    std::cerr<<"NO IMPROVMENT IN THIS CASE"<<std::endl;
  }
  
  std::cerr<<"exit gradientDescent() dist is now:"<<dist <<std::endl;  
  return dist;
}

