#ifndef MMVL_RANSACPOSEESTIMATION_H_INCLUDED
#define MMVL_RANSACPOSEESTIMATION_H_INCLUDED

#include <iostream>

using std::cout;
using std::cerr;
using std::endl;

#include <algorithm>
#include <vector>
#include "PoseEstimation.h"

#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/io.hpp>

typedef boost::numeric::ublas::vector<double> vec;
typedef boost::numeric::ublas::matrix<double> mat;

  struct count// : public generator_function<int>
    {
      count() : current(0) {}
      int current;
      int operator()(void) {return current++;}
    };
  
  class Select : public std::unary_function<int, void>
  {
  private:
    std::vector<vec> &modelPoints;
    std::vector<vec> &imagePoints;
  public:
    Select(std::vector<vec> &modelPoints,  std::vector<vec> &imagePoints)
      :modelPoints(modelPoints),imagePoints(imagePoints){}
  
    void operator() (int x)
    {
      modelPointsSample.push_back(modelPoints[x]);
      imagePointsSample.push_back(imagePoints[x]);
    }

    void clear(void)
    {
      modelPointsSample.clear();
      imagePointsSample.clear();
    }
    
    std::vector<vec> modelPointsSample;
    std::vector<vec> imagePointsSample;  
  };

/**
 *RANSAC might be a confusing name
 *This class just select a number of p>=3 pair of points from
 *a set of n pair of corresponding points and use the PoseEstimation
 *class to find the pose associated with this choice.
 *
 *Improve doc, clean code
 *
 * The estimation is done nb_estimation times and estimated transformations
 * can be found in the vector estimatedTransformations
 *
 * Time-stamp: <2004-12-14 18:24:51 engmb>
 * Copyright (C) 2004 by Manuel Boissenin
 *
 * @author Manuel Boissenin
 * @date 14/12/2004
 *
 ***********************************************************************/

class RANSACPoseEstimation{
public:
 
  std::vector<vec> estimatedTransformations;
  
  RANSACPoseEstimation(std::vector<vec>& modelPoints, std::vector<vec>& imagePoints, vec &calibration_param, vec &transformationEstimation)  
    :modelPoints(modelPoints),imagePoints(imagePoints), calib_param(calibration_param),
    transformationEstimation(transformationEstimation){}

  
  void estimatePose(int nb_estimation, int nb_point)
  {
    std::vector<int> range;
    std::back_insert_iterator<std::vector<int> > rg_it(range);
    generate_n(rg_it , 10, count());

    
    for (int i = 0; i < nb_estimation; i++)
      {
	//determine the 3 correspondant point that will be use
	std::vector<int> sample(nb_point);
	std::random_shuffle(range.begin(), range.end());
	std::copy(range.begin(), range.begin() + sample.size(), sample.begin());
#ifndef NDEBUG
	cerr<<"Selected pairs of corresponding points are: "<<endl;
	std::copy(sample.begin(), sample.end(), std::ostream_iterator<int>(cerr, " "));
#endif
	//!!!problem with constructor??
	Select select = std::for_each(sample.begin(), sample.end(),  Select(modelPoints, imagePoints));

	//Estimate the pose  //!!!speed issue to create an object so often?
		
	PoseEstimation poseEstimator(transformationEstimation ,select.modelPointsSample, select.imagePointsSample, calib_param);
	poseEstimator.estimateTransformation();
	estimatedTransformations.push_back(poseEstimator.transfEstim);
      }
    
    //display result
#ifndef NDEBUG
    cerr<<"Estimated transformations: "<<endl;
    std::copy(estimatedTransformations.begin(), estimatedTransformations.end(), std::ostream_iterator<vec >(cerr, "\n"));
#endif
  }
  
private:
  std::vector<vec>& modelPoints;
  std::vector<vec>& imagePoints;
  vec &calib_param;
  vec &transformationEstimation; 
};


#endif
