#ifndef MMVL_TRANSFORMATION_H_INCLUDED
#define MMVL_TRANSFORMATION_H_INCLUDED

#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/io.hpp>

typedef boost::numeric::ublas::vector<double> vec;
typedef boost::numeric::ublas::matrix<double> mat;


/**
 *Rotate and translate a 3D point. 
 *
 *
 *
 *
 * Time-stamp: <2004-12-14 16:44:26 engmb>
 * Copyright (C) 2004 by Manuel Boissenin
 *
 * @author Manuel Boissenin
 * @date 14/12/2004
 *
 ***********************************************************************/

class Transformation
{
public:
  mat rotation;
  vec translation;
  
  Transformation():
    rotation(3,3),translation(3){}

  //rotation around x then around y and then around z
  //angle should be given in radian
  //(x,y,z) frame is left handed frame which orientation is associated with the image frame
  //z outside the screen, x toward the right and y towards the  bottom
  void set(const vec &transformation)
  {
    double x = transformation(3);
    double y = transformation(4);
    double z = transformation(5);

    translation(0) = transformation(0);
    translation(1) = transformation(1);
    translation(2) = transformation(2);
    
    rotation(0,0) = cos(y)*cos(z);
    rotation(0,1) = sin(x)*sin(y)*cos(z) - cos(x)*sin(z);
    rotation(0,2) = cos(x)*sin(y)*cos(z) + sin(x)*sin(z);
    rotation(1,0) = cos(y)*sin(z);
    rotation(1,1) = sin(x)*sin(y)*sin(z) + cos(x)*cos(z);
    rotation(1,2) = cos(x)*sin(y)*sin(z) - sin(x)*cos(z);
    rotation(2,0) = 0-sin(y);
    rotation(2,1) = sin(x)*cos(y);
    rotation(2,2) = cos(x)*cos(y);
  }

  //rotate and translate a point according to the transfomation
  vec move(const vec &point)
  {
    return prod(rotation, point) + translation;
  }
};


#endif
