//
// Example application that uses rotationChamferCorrelationEdgeMap
// Bala Amavasai <bala@amavasai.org>
//
// $Header: /cvs/mimas2/examples/rotateChamferMatch/main.cc,v 1.1.1.1 2005/08/09 15:37:40 engmb Exp $
//


#include <iostream>
#include <cmath>
#include <ctime>
#include <fstream>
#include "image.h"
#include "image_fileinput.h"
#include "image_mesaoutput.h"
#include "image_funcs.h"
#include "canny.h"
#include "image_draw.h"

using namespace std;
using namespace mimas;

int main(int argc, char **argv)
{
  int retVal = 0;
  try {
    x11_display display;
    image_mesaoutput< unsigned char > window( &display );
	image<int>
     inimage,
     object,
     originalobject,
     rotoriginalobject,
     original;
	canny<int> canny;

	time_t starttime;// , endtime;

   ifstream inimagefile( "firstSeq0063.pgm", ios::binary );
   inimagefile >> inimage;
   original = inimage;
   ifstream objectfile( "small-lens.pgm", ios::binary );
   objectfile >> object;
   originalobject = object;

//	canny.setCannyAutoThre0(false);
	canny.setCannyStdDev(1.4);
//	canny.setCannyLowThres(1);
//	canny.setCannyHighThres(90);


	// Edge detect and perform search on lower resolution
	canny.getEdges(inimage);
	halfResolutionEdgePyramid(inimage);
	window << inimage << mimas::pause();

	canny.getEdges(object);
	halfResolutionEdgePyramid(object);
	window << object << mimas::pause();
	
	starttime=time(NULL);
	cerr << "Searching for object";

	int xmin, ymin, minangle;
	float mincorr;
	int resfactor=2; // resolution multiply factor e.g. if halfResolution is run twice than this value is 4

	// Do Chamfer matching
	rotationChamferCorrelationEdgeMap(inimage,object,0,359,10,xmin,ymin,minangle,mincorr);

	// Output some information	
	cout << "...object found (I think!). Time = " << time(NULL)-starttime << " seconds"
	    << endl; 
	cout << "(x,y)=" << xmin << "," << ymin << "   angle=" << minangle << " corrval=" << mincorr << " " << endl;

	
	// Draw an X at the point that object is found
	drawLine(original, xmin*resfactor,ymin*resfactor,resfactor*(xmin+object.getWidth()),resfactor*(ymin+object.getHeight()),63);
	drawLine(original, resfactor*(xmin+object.getWidth()),resfactor*ymin,resfactor*xmin,resfactor*(ymin+object.getHeight()),63);


	// superimpose object on scene
	canny.getEdges(originalobject);
	paethRotateRadian(originalobject,rotoriginalobject, (float)minangle*M_PI/180.0);
	for (int j=0; j<originalobject.getHeight(); j++)
	   for (int i=0; i<originalobject.getWidth(); i++)
	       if (rotoriginalobject.getPixel(i,j)==255)
		   original.setPixel(resfactor*xmin+i,resfactor*ymin+j,255);

   window << original << mimas::pause();
  } catch ( exception &e ) {
    cerr << e.what() << endl;
    retVal = 1;
  };
  return retVal;
}

