//
// Colour segmentation using hashmaps
// @author Bala Amavasai (bala@amavasai.org)
// @date Wed Mar 16 18:17:19 GMT 2005
// $Header: /cvs/mimas2/examples/segmentHashmap/main.cc,v 1.1.1.1 2005/08/09 15:37:45 engmb Exp $
//

#include <fstream>
#include <iostream>
#include "segment_hashmap_tool.h"
#include "image_fileinput.h"
#include "image_mesaoutput.h"
#include "rgba.h"

using namespace mimas;
using namespace std;

int main(int argc, char **argv)
{
	int retVal = 0;
	try {
	  x11_display display;

		MMERROR( argc >= 4, mimasexception, ,
				"Usage:  segmentHashmap <model-colour-image> <scene-colour-image> <threshold>" << endl << "        or please see the runMe file for details of how to run this app" );

		cout << endl;
		cout << "Matching colours found in model image to scene image" << endl; 
		cout << endl;

		segment_hashmap_tool seg;
		
		// load in the image that contains sample colours
		image< rgba< unsigned char > > im;
		ifstream imFile( argv[1], ios::binary ); imFile >> im;

		// load the image to be segmented
		image< rgba< unsigned char > > sim;
		ifstream simFile( argv[2], ios::binary ); simFile >> sim;

		// store the colours from the first image
		seg.storeColours(im);
		
		// segment the image
		image< rgba< unsigned char > > simout;
		seg.segment(sim, simout, atoi(argv[3]));

		image_mesaoutput< rgba< unsigned char > > window( &display );
		window << simout << mimas::pause();
		
	} catch ( exception &e ) {
		cerr << e.what() << endl;
		retVal = 1;
	};
	return retVal;
}
