/**the position of the object is its center expressed
in the image coordinate system
*/  

#include "stencil_tracking.h"
#include <GL/glut.h>
#include <image_funcs.h>
#include <filter.h>
#include <fstream>
#include <image_fileinput.h>
#include <image_loader.h>

using namespace boost;
using namespace boost::numeric;
using namespace mimas;
using namespace std;

//typedef boost::numeric::ublas::vector<int> Feature;

//To extract the feature of the image
//and reproject them on the frame of the 
//previous location 

class object_position
{
public:
  int x;
  int y;
  
  //  extract_features(const image<T>)
};


void dxvec( short int gx, short int gy, short int w,
            signed char &dx, short int &dy )
{
  short int
    agx = gx < 0 ? -gx : gx,
    agy = gy < 0 ? -gy : gy;
  if ( agy < agx ) {
    if ( ( agy << 1 ) < agx )
      dy = 0;
    else
      dy = gy < 0 ? -w : w;
    dx = gx < 0 ? -1 : 1;
  } else {
    if ( ( agx << 1 ) < agy )
      dx = 0;
    else
      dx = gx < 0 ? -1 : 1;
    dy = gy < 0 ? -w : w;
  };
}

image< bool > edgeDetect( const image< unsigned char > &m,
                             short int threshold )
{
  int thresholdSqr = (int)threshold * threshold;

  image< short int > gradientX, gradientY;

  filter<short int>::gradX(image<short int>(m) , gradientX) ;
  filter<short int>::gradY(image<short int>(m) , gradientY);
  
  image<int> gradientSqr = sumSquares(gradientX, gradientY);

  image< bool > retVal; retVal.init( m.getWidth(), m.getHeight() );

  short int w = (short)m.getWidth();
  const int *pgSqr = gradientSqr.rawData().data() + w + 1;
  const short int
    *pgX = gradientX.rawData().data() + w + 1,
    *pgY = gradientY.rawData().data() + w + 1;
  bool *pr = retVal.rawData().data() + w + 1;

  for ( int y=1; y<m.getHeight()-1; y++ ) {
    for ( int x=1; x<m.getWidth()-1; x++ ) {

      signed char dx;
      short int dy;

      dxvec( *pgX, *pgY, m.getWidth(), dx, dy );

      int g = *pgSqr;
      if ( g >= thresholdSqr && 
           g >= pgSqr[ - ( dy + dx ) ] &&
           g >= pgSqr[ + ( dy + dx ) ] )
        *pr = true;

      pgSqr++;
      pgX++;
      pgY++;
      pr++;
    };
    pgSqr += 2;
    pgX += 2;
    pgY += 2;
    pr += 2;
  };

  return retVal;
}

vector< ublas::vector< int > > featurePoints
   ( const image< bool > &binary )
{
  vector< ublas::vector< int > > retVal;
  const bool *p = binary.rawData().data();
  for ( int y=0; y<(signed)binary.getHeight(); y++ )
    for ( int x=0; x<(signed)binary.getWidth(); x++ )
      if ( *p++ ) {
        ublas::vector< int > e( 2 );
        e[0] = x;
        e[1] = y;
        retVal.push_back( e );
      };
  return retVal;
}


std::vector<Feature>
create_model( int model_width, int model_height,
	      int center_x, int center_y,
	      string image_name, int min_gradient)
{
  //Features are first extracted then expressed on the appropriate frame
  //and the one that are supposed to belong to the object are selected.
  std::vector< Feature > model;
  image<unsigned char > image; 
  std::ifstream file(image_name.c_str(), std::ios::binary);
  file >> image;

#ifndef NDEBUG  
  QTime t1;    
  t1.start();      
#endif

  std::vector< Feature> points = featurePoints( edgeDetect( image, min_gradient ) );
  
#ifndef NDEBUG  
  std::cerr<<"Time to extract the feature: "<<t1.restart()<<std::endl;
#endif

  
  for( std::vector< Feature>::iterator feature_it = points.begin();
       feature_it != points.end(); feature_it++) {
    
    Feature mapped(2);
    
    mapped(0) = (*feature_it)(0) - center_x;
    //mapped(1) = -(*feature_it)(1) + center_y; //if we want to reverse the y-axis direction
    mapped(1) = (*feature_it)(1) - center_y;
    
    //eliminate features that are not in the object boundary
    if( mapped(0) >= -model_width/2  && mapped(0) < model_width/2 &&
	mapped(1) > -model_height/2 && mapped(1) <= model_height/2 ) 
      {
	model.push_back(mapped);
      }
  }

return model;
}

std::vector<Feature>
get_features( int model_width, int model_height,
	      int center_x, int center_y,
	      image<unsigned char> image, int min_gradient)
{
  //Features are first extracted then expressed on the appropriate frame
  //and the one that are supposed to belong to the object are selected.
  std::vector< Feature > model;

#ifndef NDEBUG  
  QTime t1;    
  t1.start();      
#endif

  std::vector< Feature> points = featurePoints( edgeDetect( image, min_gradient ) );
  
#ifndef NDEBUG  
  std::cerr<<"Time to extract the feature: "<<t1.restart()<<std::endl;
#endif

  
  for( std::vector< Feature>::iterator feature_it = points.begin();
       feature_it != points.end(); feature_it++) {
    
    Feature mapped(2);
    
    mapped(0) = (*feature_it)(0) - center_x;
    //mapped(1) = -(*feature_it)(1) + center_y; //if we want to reverse the y-axis direction
    mapped(1) = (*feature_it)(1) - center_y;
    
    //eliminate features that are not in the object boundary
    if( mapped(0) >= -model_width/2  && mapped(0) < model_width/2 &&
	mapped(1) > -model_height/2 && mapped(1) <= model_height/2 ) 
      {
	model.push_back(mapped);
      }
  }

return model;
}

void display_features(std::vector<Feature> f, x11_display *disp, int border = 3)
{
  image_mesaoutput<unsigned char > display(disp);
  image<unsigned char> image;
  //Find the size of the image:
  int w_min = f[0](0);
  int w_max = f[0](0);
  int h_min = f[0](1);
  int h_max = f[0](1);

  for(std::vector<Feature>::iterator it = f.begin(); it != f.end() ; it++)
    {
      if ( (*it)(0) < w_min)
	w_min = (*it)(0);
      if ( (*it)(0) > w_max)
	w_max = (*it)(0);
      if ( (*it)(1) < h_min)
	h_min = (*it)(1);
      if ( (*it)(1) > h_max)
	h_max = (*it)(1);
    }

  image.init(w_max - w_min + 2 * border, h_max - h_min + 2 * border);
  //write on the image the features

  for(std::vector<Feature>::iterator it = f.begin(); it != f.end() ; it++)
    {
      image.pixel((*it)(0) - w_min + border, (*it)(1) - h_min + border) = 255;
    }
   display << image<< mimas::pause();
}

int main(int argc, char** argv)
{
  int retVal = 0;
  try {
    glutInit( &argc, argv );
    x11_display disp;    
    image_mesaoutput<unsigned char > display( &disp );

    //begin load parametres

    string model_image = argv[11];
    string base_name_image = argv[12];
    
    image<unsigned char> current_image;

    mimas::hf::image_loader<unsigned char> im_ld(current_image , mimas::hf::image_loader<unsigned char>::set(base_name_image, 99,".pgm",4,0));
    int threshold = atoi(argv[13]);

    int model_width = atoi(argv[1]);
    int model_height = atoi(argv[2]);
    int center_x = atoi(argv[3]);
    int center_y = atoi(argv[4]);


    object_position current_position;
    current_position.x = center_x; 
    current_position.y = center_y;
    //    double theta = atof();
    
    //int min_gradient = atoi();
      
    int x_min = atoi(argv[5]);
    int x_max = atoi(argv[6]);
    int x_sub = atoi(argv[7]);
  
    int y_min = atoi(argv[8]);
    int y_max = atoi(argv[9]);
    int y_sub = atoi(argv[10]);
  
    //float theta_min = (float)atof();
    //float theta_max = (float)atof();
    //int theta_sub = atoi();

    std::vector< Feature > model = create_model(model_width, model_height,
						center_x, center_y, model_image, threshold);

#ifndef NDEBUG
    display_features(model, &disp);
#endif
    //the class can treat more complex model (stack of images).
   
    std::vector< std::vector<Feature> > model_stack;
    model_stack.push_back(model);

    stencil_tracking tracker( model_stack, model_width, model_height, 40, &disp,
			    x_min,x_max,x_sub,
			    y_min,y_max,y_sub);
    //					     theta_min,theta_max,theta_sub) ;
        
    tracker.fill_image_space();


    while(true)
      {
	im_ld.next();
	//we are not creating again a model but the function is convenient to 
	//extract the feature of the image we need
	std::vector< Feature > centered_features = get_features(model_width + 40, model_height + 40,
						    current_position.x, current_position.y, current_image, threshold);

#ifndef NDEBUG
	display_features(centered_features, &disp);
#endif

	Transformation result = tracker.track(centered_features);
	float percentage = float(result.votes) / tracker.number_of_features[result.intervals[3].value];
#ifndef NDEBUG
	cerr<<"Indicator of success is "<<percentage<<endl;
#endif

	current_position.x += (int)( (result.intervals[0].lower_bound + result.intervals[0].upper_bound) / 2.0 );
	current_position.y += (int)( (result.intervals[1].lower_bound + result.intervals[1].upper_bound) / 2.0 );

	current_image.setPixel(current_position.x, current_position.y, 255);
	display << current_image << mimas::pause();

      }
  } catch ( exception &e )
    {
      cerr << "An exception occurred: " <<  e.what() << endl;
      retVal = 1;
    }
  return retVal;
}

 

