/**
 * 4-D of freedom 
 *
 *
 *
 *
 * Time-stamp: <05/10/19 21:06:11 engmb>
 * Copyright (C) 2005 by Manuel Boissenin
 *
 * @author Manuel Boissenin
 * @date 17/01/2005
 ***********************************************************************/

#ifndef NDEBUG
#include <boost/multi_array.hpp>
#endif

#include "stencil_tracking.h"

stencil_tracking::stencil_tracking( const std::vector< std::vector<Feature> > &feature_images,
				int model_width, int model_height, int K ,
				mimas::x11_display *disp,
				int x_min, int x_max, int x_sub,
				int y_min, int y_max, int y_sub,
				float theta_min, float theta_max, int theta_sub,
				int nb_thread):
  model_width(model_width),
  model_height(model_height),
  x_sub(x_sub), y_sub(y_sub), theta_sub(theta_sub),
  z_max(feature_images.size()),
  feature_images(feature_images),
  transformation_space(boost::extents[x_sub][y_sub][theta_sub][z_max]),
  nb_thread(nb_thread),barrier1(nb_thread + 1), barrier2(nb_thread + 1),
  current_features_to_track(*(new std::vector<Feature>))  //not elegant + memory leak...
  , display(disp) 
{
  float x_size = float(x_max - x_min)/ x_sub; //check this is not rounded
  float y_size = float(y_max - y_min)/ y_sub; 
  float theta_size = float(theta_max - theta_min)/ theta_sub; 

  //For each thread allocate space for their associate image space
  for (int i = 0; i < nb_thread; i++)
    {
      boost::multi_array<std::vector<Transformation *>,2> *image_space = 
	new boost::multi_array<std::vector<Transformation *>,2>(boost::extents[model_width + (x_max - x_min) + K][model_height + (y_max - y_min) + K]);
      image_space_vec.push_back(image_space);
    }

  
  for(int i = 0; i < nb_thread; i++)
    {
      threads.create_thread(track_t(*image_space_vec[0] , transf_it, current_features_to_track, &barrier1, &barrier2));
    }
  

  //Fill the transformation space with its values
  for(idx_ts x = 0; x < x_sub; x++)
    for(idx_ts y = 0; y < y_sub; y++)
      for(idx_ts theta = 0; theta  < theta_sub; theta++)
	for(idx_ts z = 0; z < z_max; z++)
	  {
	    //add an error to see if we get more hit
	    float error = 0.;
	    std::vector<Interval> tmp_interv;
	    tmp_interv.push_back(Interval(x_min + x * x_size - error, x_min + (1+x) * x_size + error));
	    tmp_interv.push_back(Interval(y_min + y * y_size - error, y_min + (1+y) * y_size + error));
	    tmp_interv.push_back(Interval(theta_min + theta * theta_size - error, theta_min + (1+theta)*theta_size + error));

	    //the stack is treated slighly differently
	    tmp_interv.push_back(Interval(z));

	    Transformation tmp_transf;
	    tmp_transf.intervals = tmp_interv;

	    transformation_space[x][y][theta][z] = tmp_transf; 
	  }
  for( std::vector< std::vector<Feature> >::const_iterator image_it =
         feature_images.begin(); image_it != feature_images.end(); ++image_it )
    number_of_features.push_back((*image_it).size());
}
  

/*
  Pseudo-code 

  fill_image_space()
  {
  for each transformation
     for each feature of the image corresponding to the transformation{
         calculate locus of point according to the sub-transformed space;      
         add transformation reference to this locus to the image_space array;
     }
  }

this function isn't called by the constructor in the hope later to serialize the
tracker (it is very long to calculate the data structure)
*/

void stencil_tracking::fill_image_space(void){
#ifndef NDEBUG
  unsigned n = 0;
#endif

  int switch_p = 0;//for parallelisation

  for(idx_ts theta = 0; theta  < theta_sub; theta++) {

    std::vector< std::pair< float, float > > rotations;

    //!!!!check we don't have trouble when there is no rotation

    // Pre-compute rotations to save time.
    Interval *angles = &transformation_space[0][0][theta][0].intervals[2];
    for ( float t = angles->lower_bound; t<= angles->upper_bound; t++ ) {
      float radian = t * M_PI / 180.0;
      rotations.push_back( std::make_pair( cos( radian ), sin( radian ) ) );
    };
    for(idx_ts x = 0; x < x_sub; x++)
      for(idx_ts y = 0; y < y_sub; y++)
        for(idx_ts z = 0; z < z_max; z++) {

	  switch_p++;
          Transformation *tmp_transf = &transformation_space[x][y][theta][z];
          // for(unsigned int n=0; n < transformation_space.num_elements(); n++) {
	  // Transformation tmp_transf =  transformation_space.data()[n];
	  //intervals[3] correspond to the number of the image.
	  
	  //!!!!!!!!!!!!Let's create an intermediate stencil here that will have no repetition of references

	  for( std::vector<Feature>::const_iterator feature_it = (feature_images.begin() + tmp_transf->intervals[3].value)->begin();
	       feature_it != (feature_images.begin() + tmp_transf->intervals[3].value)->end();
	       feature_it++)
	    {
	      //calculate the locus for this feature for this transformation
	      //lets start with rotation we assume !!!that
	      //leap of one degre won't make us lose any pixel
	      //we use a set of feature position avoiding repetition

	      std::set<Feature, ltfeature> rotated_locus;

	      //!!round value to find the integer box
	      /*	      std::vector< std::pair< float, float > >::const_iterator rotation_index = rotations.begin();
	      for(float t = tmp_transf->intervals[2].lower_bound; t <= tmp_transf->intervals[2].upper_bound; t++)
	      */
	      for(std::vector< std::pair< float, float > >::const_iterator rotation_index = rotations.begin();
		  rotation_index != rotations.end(); rotation_index++)
		{
		  Feature tmp_feature(2);
		  tmp_feature(0) =
		    (int)( (*feature_it)(0) * rotation_index->first -
			   (*feature_it)(1) * rotation_index->second );
		  tmp_feature(1) =
		    (int)( (*feature_it)(0) * rotation_index->second +
			   (*feature_it)(1) * rotation_index->first );
		  rotated_locus.insert( tmp_feature );
		  //		  rotation_index++;
		}
	    
	      std::set<Feature, ltfeature> x_trans_rot_locus;
	      for(int x = int(tmp_transf->intervals[0].lower_bound); x <= tmp_transf->intervals[0].upper_bound; x++)
		{
		  for( std::set<Feature, ltfeature>::const_iterator feat_it = rotated_locus.begin(); feat_it != rotated_locus.end(); feat_it++)
		    {
		      Feature tmp_feature(2);
		      tmp_feature(0) = (*feat_it)(0) + x;
		      tmp_feature(1) = (*feat_it)(1);
		      x_trans_rot_locus.insert(tmp_feature);
		    }
		}

	      std::set<Feature, ltfeature> xy_trans_rot_locus;
	      for(int y = int(tmp_transf->intervals[1].lower_bound); y <= tmp_transf->intervals[1].upper_bound; y++)
		{
		  for( std::set<Feature, ltfeature>::const_iterator feat_it =
			 x_trans_rot_locus.begin();
		       feat_it != x_trans_rot_locus.end(); feat_it++)
		    {
		      Feature tmp_feature(2);
		      tmp_feature(0) = (*feat_it)(0);
		      tmp_feature(1) = (*feat_it)(1) + y;
		      xy_trans_rot_locus.insert(tmp_feature);
		    }
		}

#ifndef NDEBUG	
	      //display features to check that we have the right points
	      /*  multi_array<bool,2> model(extents[image_space.shape()[1]][image_space.shape()[0]]);
		  for(std::set<Feature, ltfeature>::const_iterator feat_it = xy_trans_rot_locus.begin(); feat_it != xy_trans_rot_locus.end(); feat_it++)
		  {
		  model[image_space.shape()[1]/2 -(*feat_it)(1)][(*feat_it)(0) + image_space.shape()[0] /2 ] = true; 
		  }
		  display << multi_array_to_image( tobw( model ) )<<mimas::delay(500);
	      */
#endif
	      //for parallelisation we fill a stencil for each image space
	      //fill the image space with the pointer to the transformation
	    
	      
	      // }
	  for( std::set<Feature, ltfeature>::const_iterator feat_it = xy_trans_rot_locus.begin();
	       feat_it != xy_trans_rot_locus.end();
	       feat_it++)
	    {
	      //We are using an image to store the stencil
	      //coordinate must be shift so they are both positive
	      int x = (*feat_it)(0) + (*image_space_vec[0]).shape()[0]/2;
	      int y = (*feat_it)(1) + (*image_space_vec[0]).shape()[1]/2;
	      
	      //we don't want to add twice the same transform if
	      //the same sub-transformation bring two points
	      //to the same place
	      
	      //!!!!!!!here with threads does the trick of checking the last transf work?	      
	      
	      if(((*image_space_vec[switch_p % nb_thread])[x][y].empty()) || ((*image_space_vec[switch_p % nb_thread])[x][y].back() != tmp_transf))
		{
		  (*image_space_vec[switch_p % nb_thread])[x][y].push_back(tmp_transf);
		}	      
	    }
	  }
#ifndef NDEBUG	
	  n++;
	  if(!(n%3)) //prime number to avoid get trapped on a pattern corresponding to the same model images.
	    {		
	      //display image space for this transformation
	      //	      boost::multi_array<bool,2> transf_im(boost::extents[image_space_vec[0]->shape()[1]][image_space_vec[0]->shape()[0]]);
	      mimas::image<unsigned char> transf_im;
	      transf_im.init(image_space_vec[0]->shape()[1],image_space_vec[0]->shape()[0]);

	      for(idx_is x = 0; (unsigned int)(x) < image_space_vec[0]->shape()[0] ; x++)
		for(idx_is y = 0;(unsigned int) (y) < image_space_vec[0]->shape()[1] ; y++)
		  if((!(*image_space_vec[n % nb_thread])[x][y].empty()) && ((*image_space_vec[n % nb_thread])[x][y].back() == tmp_transf ))
		    {
		      transf_im.pixel(x,y) = 255; 
		    }
	      display <<  transf_im << mimas::pause();
	    }	  
#endif	 
	}
  }

#ifndef NDEBUG 
  //check the size of the image_space
  int nb_element = 0;
  for (int i = 0; i < nb_thread; i++)
    {
      for(unsigned int x = 0; x < image_space_vec[i]->shape()[0]; x++)
	for (unsigned int y = 0; y < image_space_vec[i]->shape()[1]; y++)
	  {      
	    nb_element += (*image_space_vec[i])[x][y].size();
	  }

      std::cerr<< "The image space for the thread "<< i <<"has : "<<nb_element <<" elements"<< std::endl;
      nb_element = 0;
    }
#endif     
}

Transformation stencil_tracking::track(std::vector<Feature> &features)
{
#ifndef NDEBUG 
  QTime t;    
  t.start();
  std::cerr<<"Begining of the tracking function"<<std::endl;
#endif

  //reinitialize the number of vote
  for(unsigned int n = 0; n < transformation_space.num_elements(); n++)
    transformation_space.data()[n].votes = 0;

  current_features_to_track = features;
  //make the threads wait for the previous line to be done
  barrier1.wait();
  //Synchonise the treads so they all finish their job before
  //looking fo a maximum
  barrier2.wait();
  
  Transformation max_element = *(std::max_element(transformation_space.data(),
						  transformation_space.data()+ transformation_space.num_elements(),
						  lttransf()));

#ifndef NDEBUG 
  std::cerr<<"from track()"<<std::endl;
  max_element.debug();
  std::cerr<<"Number of millisecond to track one object in this image: "<< t.restart()<< std::endl;
#endif
  return max_element;
}
