//
// Example application using wiggle filter
// stumeikle Wed May 22 18:56:12 2002
//
//


#include <iostream>
#include <vector>
#include <fstream>
#include "image.h"
#include "image_fileinput.h"
#include "image_mesaoutput.h"
#include "image_funcs.h"
#include "canny_tool.h"
#include "filter.h"
#include "corner_tool.h"
#include "prop.h"
#include "edge_string_op.h"

using namespace mimas;
using namespace std;

int main(int argc, char **argv)
{
  int retVal = 0;
  try {
    x11_display display;
    image_mesaoutput< rgba< unsigned char > > window( &display );
    image<int> 		someimage;
    canny_tool<int>		canny;
    edge_string_ptr_list	edge_strings;

    ifstream file( argv[1], ios::binary );
    file >> someimage;
    
    //(1) first run canny to extract a set of string_el strings
    edge_strings=canny.findEdgeStrings( someimage );
  
    //(2) then create an image to contain them for visualisation
    property_image	pim;
    // pim.setOwnership(false);
    pim.init( someimage.getWidth(), someimage.getHeight());
    pim.addStrings( edge_strings );    
    window << pim << mimas::pause();
    
    //(3) now filter the wiggles / texture / noise
    pim.clear();
    pim.init( someimage.getWidth(), someimage.getHeight());
    edge_string_op::filterWiggles( edge_strings, 0.2, 10, 35.0 );
    pim.addStrings( edge_strings );
    window << pim << mimas::pause();

    //(4) delete the strings
    // edge_string_ptr_list::iterator sit;
    // for(sit=edge_strings.begin();sit!=edge_strings.end();++sit)
    // delete (*sit);
  } catch ( exception &e ) {
    cerr << e.what() << endl;
    retVal = 1;
  };
  return retVal;

}


