#ifndef CGH_WRAPPER_H
#define CGH_WRAPPER_H

#include <boost/numeric/ublas/vector.hpp>
#include <boost/shared_ptr.hpp>
#include "cgh.h"

namespace mimas {

class cgh_wrapper;

///
typedef boost::shared_ptr< cgh_wrapper > cgh_wrapper_ptr;
	
/** a wrapper class for each cgh extracted from an image.
 * of which image the cgh was extracted from and where it was
 * centered. vars are public because this is just a simple
 * struct-like package really.
 *
 * @author Stuart Meikle (stu@stumeikle.org)
 * @date Sun May 26 11:03:14 2002
 * $Header: /cvs/mimas2/include/cgh_wrapper.h,v 1.1.1.1 2005/08/09 15:37:45 engmb Exp $
 */
class cgh_wrapper: public object
{

public:
    typedef boost::numeric::ublas::vector< double > Vector;
    Vector position;///< position of the corner/poi
    double		contrast;///< measured contrast (from curve-fitting input image)
    cgh_ptr cgh;    ///< pointer to the associated cgh
    
    //additional vars used in matching:
    double		match_score;///< when compared to another cgh, best score acheived
    cgh_wrapper_ptr best_match;///< ptr to the winning other cgh
    long		best_match_num;///< index of the other cgh. used when database of images used

      cgh_wrapper() 	{// cgh=NULL;  
      // best_match=NULL;
           match_score=0;
			  best_match_num=-1;}
      // ~cgh_wrapper() 	{ if (cgh)	delete cgh; }
};

///
typedef std::list< cgh_wrapper_ptr > cgh_wrapper_ptr_list;

}

#endif
