/** This is binary search using cumulative probabilities to pick a base
   hypothesis.  More details
*/

template< typename Hypothesis_T>
int condensation<Hypothesis_T>::pick_base_hypothesis(void)
{
  calculate_cumulative();
  double max_cumul_weight = cumulative_weight.back();

  //if all hypothesis have a zero weight the track is supposed to be lost
  //just warn the user
  if(max_cumul_weight == 0)
    {
      std::cerr<<"No hypothesis obtained a satisfactory measure, the track might be about to fail"<<std::endl;
      // -1 to avoid the improbable case that the uniform random is 1
      return int(Dice::uniform_random() * (hypotheses_CD.size() - 1));
    }
  
  double choice = Dice::uniform_random() * max_cumul_weight;
  int low, middle, high;

  low = 0;
  high = hypotheses_CD.size();
  
  while (high > (low+1) )
    {
      middle = (high+low) / 2;
      if (choice > cumulative_weight[middle])
	low = middle;
      else high = middle;
    }

  return low;
}

template< typename Hypothesis_T>
void condensation<Hypothesis_T>::calculate_cumulative(void)
{
  double cumul_total = 0.0;

  cumulative_weight.clear();
  cumulative_weight.push_back(cumul_total);

  //Note that the cumulative_weight vector size is nb_hypothesis + 1.

  typename std::vector<Hypothesis_T>::iterator it;
  for (it = hypotheses_CD.begin() ; it != hypotheses_CD.end(); it++) { 
    cumul_total += it->weight;   
    cumulative_weight.push_back(cumul_total);
    //Here we can calculate the max weight if needed. Can be an interesting information for
    //user of the class. To display hypothesis with colors depending on the weight ... 
    //But for the moment we want a minimalist implementation. Using inheritence will allow us to have
    //a fancy condensation class.
  }
}
