#ifndef CORRELATE_SQUARE_REGION_H
#define CORRELATE_SQUARE_REGION_H

#include <boost/numeric/ublas/vector.hpp>
#include <boost/shared_ptr.hpp>
#include <functional>
#include "image.h"
namespace mimas {
template< typename T, class Corr >
class correlate_square_region: public std::binary_function
    < boost::numeric::ublas::vector< double >,
      boost::numeric::ublas::vector< double >, double >
{
 public:
  typedef boost::numeric::ublas::vector< double > Vector;
  typedef boost::shared_ptr< image< T > > image_ptr;
  correlate_square_region( image_ptr aImage1,
                              image_ptr aImage2,
                              int aAreaDiv2,
                              Corr &aCorr ):
    image1(aImage1), image2(aImage2), areaDiv2( aAreaDiv2 ), corr(aCorr) {
      assert( image1 ); assert( image2 ); assert( areaDiv2 >= 0 );
    }
  double operator()( const Vector &a,
                     const Vector &b );
 protected:
  image_ptr image1;
  image_ptr image2;
  int areaDiv2;
  Corr corr;
};

}
#include "correlate_square_region.tcc"
#endif
