//change the name of the functions (and file) to C++ style names. (and not Java)

template<typename Hypothesis_T, typename Pixel_T>
void correlation_observer2D<Hypothesis_T, Pixel_T>::observe(Hypothesis_T &h)
{
  double result;
 
  //bounds treatments before and after.
  CLAMP(h.x, KER_W, image_private.getWidth() - KER_W); 
  CLAMP(h.y, KER_H, image_private.getHeight() - KER_H);

  result = convolvePoint(int(h.x),int(h.y));

  //correlation coeff is between -1 and 1
  //as we are not interested by negative correlation and that observe() need to 
  //provide positive results only, we can threshold the value at 0

  //fullfill the contract with filter here no need because of previous line
#ifndef NDEBUG
  if(isnan(result)){
    std::cerr<<"In correlation_observer2D::observe() there is a problem"<<std::endl;
    result = 0;
  }

  //  std::cerr<<"Result for this hypothesis is: "<< result<< std::endl;
  if (result < 0.)
    result = 0;

  image_private.setPixel(int(h.x), int(h.y), (unsigned char)(255 - result*255));
#endif

#ifndef NDEBUG
  //display the kernel image on the top-left corner of the image
  for(int x = 0; x< kernel_image.getWidth(); x++ ) 
    for(int y = 0; y < kernel_image.getHeight(); y++)
      image_private.setPixel(x,y, kernel_image.getPixel(x,y));
#endif
  
  h.weight = result;
}

template<typename Hypothesis_T, typename Pixel_T>
double correlation_observer2D<Hypothesis_T, Pixel_T>::convolvePoint(const int x,const int y) const
{
  register int i, j, Ker_Pos_X, Ker_Pos_Y;

  std::vector<int> current;
  corrcoef< std::vector< int > > corrCoef;
  
  Ker_Pos_X = x - (KER_W-1)/2;
  Ker_Pos_Y = y - (KER_H-1)/2;
	
  //THE ORDER MATTERS
  for (j = 0; j < KER_H; j++)	
    for (i = 0; i < KER_W; i++)
      {
	unsigned char temp = image_private.getPixel(Ker_Pos_X+i,Ker_Pos_Y+j);
	current.push_back(temp);
      }

  return corrCoef(current,kernel);
}

/******************************************************************
 * NAME:
 *      set_correlation_observer_members
 *
 * DESCRIPTION:
 *      Extracts the piece of image that will be used as the
 *	correlation kernel.
 *
 * PARAMETERS:
 *      
 *      x,y          - 2D coordinates of the center of the kernel
 *      KER_W        - width of the kernel
 *      KER_H        - height of the kernel
 * 
 * RETURNS:
 *	      kernel_image - array holding the extracted kernel for displaying purposes
 *            kernel     - array holding the normalized kernel
 *
 *******************************************************************/
template<typename Hypothesis_T, typename Pixel_T>
void correlation_observer2D<Hypothesis_T, Pixel_T>::set_correlation_observer_members(const int x,const int y)
{
	register int i, j, Ker_Pos_X, Ker_Pos_Y;

       	Ker_Pos_X = x - (KER_W-1)/2;
	Ker_Pos_Y = y - (KER_H-1)/2;
	
	//THE ORDER MATTERS
	for (j = 0; j < KER_H; j++)	
	  for (i = 0; i < KER_W; i++)
	    {
	      unsigned char temp = image_private.getPixel(Ker_Pos_X+i,Ker_Pos_Y+j);
	      kernel_image.setPixel(i,j,temp);
	      kernel.push_back(temp);
	    }

}
