//
// edge string funcs or edge string operators
// stumeikle Mon May  6 18:10:06 2002
//

#ifndef EDGE_STRING_OP_H
#define EDGE_STRING_OP_H

#include <boost/numeric/ublas/vector.hpp>
#include "property_image.h"
#include "edge_string.h"

namespace mimas {

//operators
class edge_string_op
{
    // just a namespace really. functions which operate on
    // edge strings.    
    //20030331... almost
    static bool				fast_mode;

    static int				shortSpur( const string_el &spur, const string_el &last, int thres, int count);
    static edge_string_ptr_list findEdgeStrings( property_image& input, int, double );
    static void 			pruneSpurs( string_el_ptr_list& junc_list, string_el_ptr_list& term_list );
    static void				pruneTees(string_el_ptr_list& junc_list, string_el_ptr_list& term_list );
    static void				pruneJuncs(string_el_ptr_list& junc_list, string_el_ptr_list& term_list );
    static void				removeJuncs( string_el_ptr_list& junc_list,
							string_el_ptr_list& term_list );
    static void				bridgeSmallGaps( string_el_ptr_list& term_list );

public:
    typedef boost::numeric::ublas::vector< double > Vector;
    static edge_string_ptr_list extractFromImage( property_image& im, int len_thres,
							  double up_edge_thres );
    static edge_string_ptr_list         findStrings( string_el_ptr_list& term_list );
    static edge_string_ptr		findEdgeString( string_el_ptr el  );
    static edge_string_ptr		findLoop( string_el_ptr el );
    static void				thresholdStrings( edge_string_ptr_list& strings,
                                             int lt, double t );
    static bool				thresholdString( const edge_string& s, int len, double t );
    static void				findContrastMaxima( edge_string& s, property_image& cim );
    static void				filterWiggles( edge_string_ptr_list& edge_strings,
					  double wiggle_thres, double len_thres, double contrast_thre );

    //fast methods for fast canny. 20030331
    static void				useFastMode() {fast_mode=true;}
    static void				useNormalMode(){fast_mode=false;}
    static edge_string_ptr_list findEdgeStrings_Fast( property_image& input, int len_thres, double thres );
    static edge_string_ptr getString_Fast(property_image& input, int len_thres, double thres,
					      	int start_x, int start_y );
    static void  			firstHalf_Fast(property_image& input, double thres,
				     		int startx, int starty, edge_string& es,
				     		bool* crossed_thres, long *len );
    static void  			secondHalf_Fast(property_image& input, double thres,
				     		int startx, int starty, edge_string& es,
				     		bool* crossed_thres, long *len );


};

};

#endif



