//
// edgel stuff
// stumeikle Sun May 12 20:30:02 2002
//
//

#ifndef EDGEL_H
#define EDGEL_H

#include <boost/numeric/ublas/vector.hpp>
#include <boost/shared_ptr.hpp>
#include "angle.h"
#include "prop.h"
#include "image_draw.h"

namespace mimas {
	
/**
 *
 * Forms part of canny edge extraction tool. Algorithms and inspiration from TINA.
 * Canny_Tool is an operator which is used to extract edge strings from
 * input images. Edge extraction method is based upon Canny's algorithm and
 * includes Canny's post processing methods. Found edges are grouped into
 * continuous strings. This class is a template class because it must operator
 * on a template class (the input image).
 *
 *
 * @author Stuart Meikle (stu@stumeikle.org)
 * @date Sun May 12 20:30:02 2002
 * $Header: /cvs/mimas2/include/edgel.h,v 1.1.1.1 2005/08/09 15:37:45 engmb Exp $
 */

class edgel : public prop		///< edgel is a kind of image property
{
    private:
   typedef boost::numeric::ublas::vector< double > Vector;
   Vector centre;		///< position of the edge w.r.t the image plane
   double strength;	///< strength of the edge in terms of contrast
   angle orientation;	///< tangent to the edge string, 0-2*M_PI i think.

    public:
   /// constructor
	edgel(void) : prop( edgel_t ), centre( 2 ) {}

	//inspectors and modifiers
	void 	setPosition( double x, double y) { centre(0) = x; centre(1) = y; }
	void	setStrength( double s ) 	{ strength = s; }
	void	setContrast( double s ) 	{ strength = s; }
	void	setOrientation( angle a )    { orientation = a; }

	const Vector &getPosition() const { return centre; }
	angle	getOrientation()	{ return orientation; }
	double		getStrength()		{ return strength; }
	double		getContrast()		{ return strength; }

	/** Draw the edgel at the specified position. If 'c' is true also show the strength.
	*   This function is called from the property_image draw function and is used
	*   for the case where we have a mixture of properties in a single image
	*/
	virtual void draw( image< rgba< unsigned char > >& output,
                      int i, int j, bool c )
	{
	    double e1x,e1y,e2x,e2y;
	    angle a = orientation;

	    //should really check to see if the orientation was set
	    //you could draw the orientation of the edgel by increasing
	    //the '1's below to a higher value.
	    e1x = centre(0) + cos(a)*1;
	    e1y = centre(1) + sin(a)*1;
	    a+=M_PI;

	    e2x = centre(0) + cos(a)*1;
	    e2y = centre(1) + sin(a)*1;
	    drawLine(output, (int) e1x, (int) e1y, (int) e2x, (int) e2y,
                        rgba< unsigned char >( (int)255, 0, 0 ) );
	}
};

typedef boost::shared_ptr< edgel > edgel_ptr;

};

#endif

