#ifndef MMVL_HYPOTHESIS2D_HH
#define MMVL_HYPOTHESIS2D_HH

#ifndef NDEBUG
#include <iostream>
#endif

#include "hypothesis.h"
#include "dice.h"

using namespace mimas;

namespace mimas{
  namespace hf{
    
    /**
     *\ingroup pf
     *@brief 2-D location of an object
     *
     *if the object has a transversal movement for instance
     *
     *************************************************************************/
    class hypothesis2D: public hypothesis
      {
      public:
	float x;
	float y;
	///  float sigmaSpan;///<Don't forget to change this value according to the desired drift
	static float sigmaSpan;///<Don't forget to change this value according to the desired drift
      
	///carefull some memory is allocated with this function !!
	// !! instead of using int(), use something to round to the nearest value 
	hypothesis2D drift(void) const
	  {
	    return hypothesis2D(int(x + Dice::gaussian_random() * sigmaSpan),
				int(y + Dice::gaussian_random() * sigmaSpan),
				weight);
	  }

	//This is used by analyse_result_mean
	//when using this function make sure you are not dividing by 0 
	//!!!might be implemented as a type trait so s.o. who don't
	//bother about AnalyseResult won't have to implement it
	hypothesis2D mean(const hypothesis2D &h)
	{
	  if(h.weight != 0)
	    {
	      float coord1 = (x * weight  + h.x * h.weight) /(weight + h.weight);
	      float coord2 = (y * weight + h.y * h.weight) /(weight + h.weight);

	      return hypothesis2D(coord1,
				  coord2,
				  weight + h.weight);
	    }
	  return *this;
	}

	float distance(const hypothesis2D &h)
	{
	  return sqrt((h.x - x) * (h.x - x) + (h.y - y) * (h.y - y));
	}

	hypothesis2D(float x = 0, float y = 0, double weight = 0)
	  :hypothesis(weight), x(x), y(y){}
      
      };
  }
} 
#endif
