template <
  template <typename> class Image_Loader_P,
  template <typename, typename> class Observer_P,
  template<typename, typename> class Analyse_result_T,
  typename Hypothesis_T,
  template <typename> class Picking_P,
  template <typename> class Compensate_P,
  typename Pixel_T
  >
void hypothesis_filter<Image_Loader_P, Observer_P, Analyse_result_T,
		      Hypothesis_T, Picking_P, Compensate_P, Pixel_T>::filter(void)
{
  // !!!use iterator instead
  int base;
  for (int n = 0; n < nb_hypothesis; ++n){
    base = Picking_P<Hypothesis_T>::pick_base_hypothesis();  ///<provided by the picking policy

    //relocate hypotheses
    hypotheses_new.push_back(hypothesis_filter_data<Hypothesis_T, Pixel_T>::hypotheses[base].drift()); ///< drift() provided by the hypothesis itself
    
    observe(hypotheses_new.back());  ///<provided by the observer policy
    
    outweigh(hypotheses_new.back());  ///<provided by the compensate policy
    
    //checks
    assert(hypotheses_new.back().weight >= 0); 
    if(isnan(hypotheses_new.back().weight)){
      cerr <<
	"This is not a number (NAN)." <<
	"You may check the value you are giving to weight the hypothesis: \n" <<
	"with your observer."<<endl;
    }
  }
  update();
}


template <
  template <typename> class Image_Loader_P,
  template <typename, typename> class Observer_P,
  template<typename, typename> class Analyse_result_T,
  typename Hypothesis_T,
  template <typename> class Picking_P,
  template <typename> class Compensate_P,
  typename Pixel_T
  >
double hypothesis_filter<Image_Loader_P, Observer_P, Analyse_result_T,
			Hypothesis_T, Picking_P, Compensate_P, Pixel_T>::max_weight(void) 
{
  double max_weight = 0;
  typename std::vector<Hypothesis_T>::iterator it;
 
  for (it = hypothesis_filter_data<Hypothesis_T, Pixel_T>::hypotheses.begin(); it != hypothesis_filter_data<Hypothesis_T, Pixel_T>::hypotheses.end(); ++it){
    if((*it)->weight > max_weight)
      max_weight = (*it)->weight;
  }
  return max_weight;
}

template <
  template <typename> class Image_Loader_P,
  template <typename, typename> class Observer_P,
  template<typename, typename> class Analyse_result_T,
  typename Hypothesis_T,
  template <typename> class Picking_P,
  template <typename> class Compensate_P,
  typename Pixel_T
  >
void hypothesis_filter<Image_Loader_P, Observer_P, Analyse_result_T,
		      Hypothesis_T, Picking_P, Compensate_P, Pixel_T>::reset(Hypothesis_T & init) 
{
  reset_hypothesis(init);
  hypotheses_new.clear();           
}

template <
  template <typename> class Image_Loader_P,
  template <typename, typename> class Observer_P,
  template<typename, typename> class Analyse_result_T,
  typename Hypothesis_T,
  template <typename> class Picking_P,
  template <typename> class Compensate_P,
  typename Pixel_T
  >
void hypothesis_filter<Image_Loader_P, Observer_P, Analyse_result_T,
		      Hypothesis_T,Picking_P, Compensate_P, Pixel_T>::reset_hypothesis(const Hypothesis_T & init)
{
  hypothesis_filter_data<Hypothesis_T, Pixel_T>::hypotheses.clear();
  for(int i = 0; i < nb_hypothesis; ++i)
    hypothesis_filter_data<Hypothesis_T, Pixel_T>::hypotheses.push_back(init.drift());
}

template <
  template <typename> class Image_Loader_P,
  template <typename, typename> class Observer_P,
  template<typename, typename> class Analyse_result_T,
  typename Hypothesis_T,
  template <typename> class Picking_P,
  template <typename> class Compensate_P,
  typename Pixel_T
  >
void hypothesis_filter<Image_Loader_P, Observer_P, Analyse_result_T,
		      Hypothesis_T, Picking_P, Compensate_P, Pixel_T>::update(void)
{
  hypothesis_filter_data<Hypothesis_T, Pixel_T>::hypotheses = hypotheses_new; 
  hypotheses_new.clear();           
}
