#ifndef IMAGE_FILESINPUT_H
#define IMAGE_FILESINPUT_H

#include <deque>
#include <string>
#include "image_fileinput.h"

namespace mimas {

/** @addtogroup imageIO
    @{ */
/** @addtogroup imageInput
    @{ */
/** Class for reading a set of image-files.
    Example of usage:
    \code
    using namespace mimas;
    int main( int argc, char *argv[] ) {
       image_filesinput< rgba< unsigned char > > input;
       for ( int i=1; i<argc; i++ )
          input.addFileName( argv[i] );
       ...
       image< rgba< unsigned char > > image;
       while ( input >> image ) {
          ...
       };
    }
    \endcode
    @see image_fileinput */
template< typename T >
class image_filesinput: public image_input< T >
{
 public:
  /// Constructor.
  explicit image_filesinput(void) {}
  /** Add a filename to the list.
      A filename is added to the end of the list. */
  void addFileName( const std::string &fileName )
  { fileNames.push_back( fileName ); }
  /** Read the next image file.
      The first filename in the list is removed and the file with this
      name is opened. The content of the file is read into the image.
      @param image Object to store image in. */
  virtual void read( image< T > &image ) throw (mimasexception);
 protected:
  /// List of filenames.
  std::deque< std::string > fileNames;
};

///@}

///@}

};

#include "image_filesinput.tcc"

#endif
