#include <sys/sem.h>
#include <sys/shm.h>

namespace mimas {

template< typename T >
image_gstreamerinput< T >::image_gstreamerinput( int _semkey,
                                                       int _shmkey )
  throw (mimasexception):
  semkey(_semkey), shmkey(_shmkey)
{
  // Get hold on semaphore.
  semid = semget( semkey, 2, 0 );
  MMERROR( semid >= 0, mimasexception, ,
           "Could not get semaphore with key \"" << semkey << "\"." );

  // Get id of shared memory.
  shmid = shmget( shmkey, 8, 0 );
  MMERROR( shmid >= 0, mimasexception, ,
           "Could not access shared memory with key \"" << shmkey << "\"." );

  // Attach to shared memory.
  shmem = shmat( shmid, NULL, 0 );
  MMERROR( shmem != NULL, mimasexception, ,
           "Couldn't attach pointer to shared memory with key \"" << shmkey
           << "\"." );
}

template< typename T >
image_gstreamerinput< T >::~image_gstreamerinput(void)
{
  // Detach from shared memory.
  shmdt( shmem );
}

};
