#include "mesa_onscreen.h"

#ifdef HAVE_OPENGL

namespace mimas {

mesa_onscreen::mesa_onscreen( x11_window *_window )
  throw (mimasexception):
  mesa_base( _window->get_display(), _window->get_visual_info() ),
  window(_window), finishOnScreen( false )
{
  // Create OpenGL rendering context for on-screen rendering.
  // Attention: Direct rendering is disabled (GL_FALSE), because it
  // doesn't work together properly with mesa_offscreen in all
  // cases (f.e. 16-bit display with NVidia).
  context = glXCreateContext( window->get_display()->get(),
                              window->get_visual_info(), 0, GL_FALSE );
  glWrap( context != NULL, "Error creating GLX-context" );

  // Attach context to the window.
  glWrap( glXMakeCurrent( window->get_display()->get(), window->get_window(),
                          context ), "Error switching GLX-context" );
}

mesa_onscreen::~mesa_onscreen(void)
{
  finish();
}

void mesa_onscreen::finish(void)
{
  if ( !finishOnScreen ) {

    glFinish();

    glXDestroyContext( window->get_display()->get(), context );
    context = NULL;
    finishOnScreen = true;
    window = NULL;

  };

  mesa_base::finish();

}

void mesa_onscreen::ortho2D(void) const
{
  ortho2D( 0, 0, window->get_width(), window->get_height() );
}

void mesa_onscreen::ortho2D( int x1, int y1, int x2, int y2 ) const
{
  assert( !finishOnScreen );
  glLoadIdentity();
  glViewport( x1, window->get_height() - y2 - 1,
	      x2 + 1 - x1, y2 + 1 - y1 );
  glOrtho( x1, x2 + 1, y2 + 1, y1, -1.0, 1.0 );
  // glOrtho( 0.0, window->get_width(), window->get_height(), 0.0, -1.0, 1.0 );
}

};

#endif

