/*
 * pol_locator_hough.cc
 *
 * see the header file for comments
 * stumeikle . Mon 04 Nov 2002 17:22:04 CET
 * $Header: /cvs/mimas2/lib/pol_locator_hough.cc,v 1.1.1.1 2005/08/09 15:37:29 engmb Exp $
 *
 */

#include "mimasconfig.h"
#include <cmath>
#include <cstdarg>
#include "pol_match.h" 
#include "pol_locator_hough.h"

namespace mimas {
using namespace std;

	//first the interface functions
	void    pol_locator_hough::init( int numdims, ... )
	{
		va_list	vlist;
		va_start(vlist, numdims );

		//get whatever parameters we want. in this case the image width and height
		//and the sigma and precision of the gauss.
		int	w,h;
		double	s,p;

		w=va_arg(vlist, int);
		h=va_arg(vlist, int);
		s=va_arg(vlist, double);
		p=va_arg(vlist, double);

		image<pol_angle>::init(w,h);
		initFastGauss(s,p);

		va_end(vlist);
	}

	void	pol_locator_hough::addEntry( double weight , ... )
	{
		va_list	vlist;
		va_start( vlist, weight );

      Vector v( 2 );
      v( 0 ) = va_arg(vlist, double);//+ox;
		v( 1 ) = va_arg(vlist, double);//+oy;
		double		a= va_arg(vlist, double);

		addGaussPointFast(v, weight, a);

		va_end(vlist);
	}

	void	pol_locator_hough::setLocation( int x, int y, double ca, double sa )
	{
	    //here's the clever bit then. we make an angle entry at a position
	    //with a weight.
	    ca*=10.0;
	    sa*=10.0;

	    int		a = (int)pixel(x,y).x+(int)ca;
	    int		b = (int)pixel(x,y).y+(int)sa;

	    pixel(x,y).x=(int16_t)a;
	    pixel(x,y).y=(int16_t)b;
	}

	int	pol_locator_hough::calcGaussBoxSize( double sigma, double accuracy )
	{
	    //code largely taken from tina src.
	    double n;

	    sigma = 2.0 * sigma * sigma;

	    for(n=1.0; ;n+=1.0)
	    {
		if (exp( -n*n/sigma )< accuracy) break;
	    }

	    return (int) n;
	}

	void	pol_locator_hough::initFastGauss( double sigma, double precision )
	{
	    // create a convolution-like kernal for a gauss blob
	    // copied from hough_transform.cc

	    if (bell_curve)	delete bell_curve;

	    int	size;
	    double sig;

	    sig = 2.0 *sigma * sigma;
	    size = calcGaussBoxSize( sigma, precision );

	    bell_curve = new image<double>;
	    bell_curve->init( (size*2)+1, (size*2)+1 );

	    for(int y=0;y<(size*2)+1;++y)
	    {
	    	for(int x=0;x<(size*2)+1;++x)
		{
		    double dx,dy,d,w;
	    
		    dx = (double)(x-size);
		    dy = (double)(y-size);
		    d  = dx*dx+dy*dy;

		    w  = exp ( -d/sig );
		    bell_curve->pixel(x,y) = w;
		}
	    }
	}

	void	pol_locator_hough::addGaussPointFast( const Vector &posn, double w, double angle )
	{
	    //modified from hough_transform
	    //as for plot point, but the bell curve must be initialised first!
	    int	size,hsize;
	    int x,y,sx, sy,o_sx,o_sy;
	    double v;

	    if (!bell_curve)	return;

	    size = bell_curve->getWidth();
	    hsize=(size-1)/2;

	    o_sx = (int)(posn(0) - (double)(hsize));
	    o_sy = (int)(posn(1) - (double)(hsize));

	    double	ca = cos(angle)*w;
	    double	sa = sin(angle)*w;

	    if (o_sx<0 || (o_sx+size)>=getWidth())
		return;
	    if (o_sy<0 || (o_sy+size)>=getHeight())
		return;

	    for(y=0, sy=o_sy;y<size;++y, ++sy)
	    {
		for(x=0, sx=o_sx;x<size;++x, ++sx)
		{
		    v=bell_curve->pixel(x,y);
		    setLocation(sx,sy,ca*v,sa*v);
		}
	    }
	}

// 		void pol_locator_hough::display( void )
// 		{
// 		    //virtual as different kinds of image may draw themselves differently
// 			int somearg=0;
// 			char **someargv=NULL;
// 			QApplication app(somearg, someargv);
// 			QImage img(getWidth(),getHeight(),QImage::Format_RGB32);

// 			double		max=0;

// 			for(int y=0;y<getHeight( );y++ )
// 				for(int x=0;x<getWidth( );x++ )
// 				{
// 				    int v = pixel(x,y).magSquared();

// 				    if ((double)v>max)
// 					max=(double)v;
// 				}

// 			max=sqrt(max);

// 			for(int y=0;y<getHeight();y++ )
// 				for(int x=0;x<getWidth();x++ )
// 				{
//               int	r,g;

// 				    r = pixel(x,y).x;
// 				    g = pixel(x,y).y;

// 				    if (r<0)	r*=-1;
// 				    if (g<0)	g*=-1;

// 				    r = (int)((double)r*250.0/max);
// 				    g = (int)((double)g*250.0/max);
// 				    //if (r>0)	r=255;
// 				    //if (g>0)	g=255;

// 				    img.setPixel( x,y, qRgb(r,g,0));
// 				}

// 			QLabel lab(0);
// 			lab.setPixmap( QPixmap::fromImage( img ) );
// 			lab.show();
// 			// app.setMainWidget(&lab);
// 			app.exec();
// 		}


	pol_match_ptr_list	pol_locator_hough::getMatches()
	{
	    //process the now full hough and return a list of the found objects and their
	    //positions
	    pol_match_ptr_list	retval;

	    //display();

	    int	x,y;
	    //(1) threshold
	    int	cutoff = (int)(thres*thres);
	    for(y=0;y<getHeight();++y)
	    {
		for(x=0;x<getWidth();++x)
		{
		    if (pixel(x,y).magSquared() < cutoff)
		    {
			pixel(x,y).x=0;
			pixel(x,y).y=0;
		    }
		}
	    }

	    //display();
	    
	    image<int>	im;
	    im.init( getWidth(), getHeight() );
	    
	    //(2) find the peaks
	    for(y=1;y<getHeight()-1;++y)
	    {
		for(x=1;x<getWidth()-1;++x)
		{
		    int	v = pixel( x, y ).magSquared();
		    
		    if (v==0)
			continue;

		    int vn= pixel(x,y-1).magSquared();
		    int vs= pixel(x,y+1).magSquared();
		    int ve= pixel(x+1,y).magSquared();
		    int vw= pixel(x-1,y).magSquared();
		    int vne= pixel(x+1,y-1).magSquared();
		    int vnw= pixel(x-1,y-1).magSquared();
		    int vse= pixel(x+1,y+1).magSquared();
		    int vsw= pixel(x-1,y+1).magSquared();

		    if (v>=vn && v>=vs && v>=ve && v>=vw &&
		        v>=vne&& v>=vnw&& v>=vse&& v>=vsw )
		    {
		    	im.pixel(x,y) = 255;

			//ok got a peak.
         pol_match_ptr o( new pol_match );
         Vector pos( 2 );
         pos( 0 ) = (double)x+0.5; pos( 1 ) = (double)y+0.5;
			o->setPosition(pos);
			
			angle		a(  (double)pixel(x,y).y, 
						    (double)pixel(x,y).x );
			
			o->setOrientation( a );

#ifndef NDEBUG			
			cerr << "Found an instance at ( " << pos(0) << "x" 
              << pos(1) 
              << " with angle " << (double)a/0.0174533 
			     << " and strength " << sqrt((double)v) << endl;
#endif
			     
			o->setStrength(sqrt((double)v));
			retval.push_back(o);

			//blank the neighbourhood in case its a blurry point.
			//no don't do this. need another hough really.
			//for(int j=y-1;j<=y+2;j++)
			//{
	    	 	//    for(int i=x-1;i<=x+2;i++)
			//	setPixel(i,j,0);
	    		//}

    		    }//if peak
		}
	    }	    

	    //im.display();
	    
	    return retval;
	}

} 

