/**
 *
 * SLAP == Straight Line APproximation tool.
 *
 * use this tool to turn edge strings into lists of straight line segments.
 * Algorithm here is based on the original by Dr xxxx (can fbc provide this 
 * reference please?). Implementation by stu. Sun 13 Oct 2002 14:45:26 CEST
 *
 *
 * Algorithm takes a parameter- anglesigma which is used to determine how the
 * edge strings should be broken into straight line segments. A high value 
 * means that only large changes in the angle of an edge string will be 
 * encorporated into a single straight line, therefore producing a very 
 * approximate set of output lines (blocky). With a lower value the straight
 * lines more closely follow the edge strings and the output will look more
 * like the input. (smoother)
 *
 *
 * @author$
 * @date$
 * $Header: /cvs/mimas2/lib/slap_tool.cc,v 1.1.1.1 2005/08/09 15:37:29 engmb Exp $
 */

#include "linalg.h"
#include "slap_tool.h"

using namespace std;
using namespace boost;

namespace mimas {

    /**
    * Return a list of straight lines from a given list of edge strings. 
    * A suitable list of edge strings is produced by, for example, the 
    * canny_tool.
    */
  line_ptr_list	slap_tool::findStraightLines( edge_string_ptr_list& es_list )
    {
	line_ptr_list geom;
    
	geom = poly_strings(es_list, anglesigma / 4.0);

	return geom;
    }
    
    line_ptr_list	slap_tool::poly_strings( edge_string_ptr_list& es_list, double thres )
    {
	//for each string in the edge_strings...
	edge_string_ptr_list::iterator	i;
	line_ptr_list			retval;
	line_ptr_list::iterator	j;
	
	for(i=es_list.begin();i!=es_list.end();++i)
	{
	    edge_string_ptr es;
	    line_ptr_list	geom;
	    
	    es = (*i);
	    geom = poly_string( *es, thres );
	    
	    //geom is now a list of straight line segments
	    //we will just add them all to the final list and then destroy the geom list
	    //The pairwise routines take a single list of straight line segments so this
	    //should be ok. maybe tomorrow things will change.
	    for(j=geom.begin();j!=geom.end();++j)
	    {
         retval.push_back( *j );
	    }
	    
	    geom.clear();
	}
	
	return retval;
    }

    line_ptr_list	slap_tool::poly_string( edge_string& es , double thres )
    {
	line_ptr_list	result;
	string_it	endit = es.end();
	
	endit--;

	poly_prox( es.begin(), endit, thres, result );
	
	return result;
    }
    
    void	slap_tool::poly_prox( string_it start_it, string_it end_it, double thres, line_ptr_list& result )
    {
	// DANGER DANGER, recursive
	double		length,max_perp;
	string_it	max_it;
	Vector s2e(2), u_s2e(2);
	string_el_ptr start, end;
	
	start = *start_it;
	end   = *end_it;
	
	//find the distance from the start to the end
	if (!start || !end)
	    return;	
	s2e = end->getPosition()-start->getPosition();

	length = norm_2( s2e );
		    
	//create a unit vector along this line
	u_s2e= unit( s2e );
	
	//go through the string and find the position with the maximum perpendicular
	//distance from the straight line connecting the start and the end positions
	string_it	i;
	max_perp=0;
	for(i=start_it;i!=end_it;++i)
	{
	    string_el_ptr sel = (*i);
	    Vector v( sel->getPosition() - start->getPosition() );
	    //project v onto the unit vector 
	    Vector		perp( 2 );
	    double		perp_mag;
	    
	    perp = v - ( u_s2e * inner_prod( v, u_s2e ) );

	    perp_mag = norm_2( perp );
	    if (perp_mag > max_perp)
	    {
		max_perp = perp_mag;
		max_it   = i;
	    }	    
	}

	//check to see if the segment is straight enough
	//here we APPROXIMATE tan(angle) and compare it to the threshold
	if (max_perp/length < thres)
	{
	    //yes, create a line segment and push it onto the return list
     line_ptr
       retval( new line(start->getPosition(), end->getPosition()) );
	    
	    result.push_back( retval );
	    return;
	}

	//no, break it down and try again, recursing...
	poly_prox( start_it, max_it , thres, result );
	poly_prox( max_it,   end_it , thres, result );
	return;
    }    
}
