#include "mimasconfig.h"

#ifdef HAVE_XALAN

#include <sstream>
#include <sax/SAXParseException.hpp>
#include <Include/PlatformDefinitions.hpp>
#include "xml_strx.h"
#include "xml_errorhandler.h"

using namespace std;
// using namespace xercesc;

// XALAN_USING_XERCES(SAXParseException);

namespace mimas {

xml_errorhandler::xml_errorhandler(void):
  errorOccurred(false)
{
}

void xml_errorhandler::warning( const SAXParseException &toCatch )
{
  // triggerError( "Warning", toCatch );
#ifndef NDEBUG
  cerr << createMessage( "Warning", toCatch ).c_str() << endl;
#endif
}

void xml_errorhandler::error( const SAXParseException &toCatch )
{
  triggerError( "Error", toCatch );
}

void xml_errorhandler::fatalError( const SAXParseException& toCatch )
{
  triggerError( "Fatal error", toCatch );
}

void xml_errorhandler::resetErrors(void)
{
  errorOccurred = false;
  message.clear();
}

void xml_errorhandler::triggerError( const std::string &type,
                                        const SAXParseException &toCatch )
{
  errorOccurred = true;
  ostringstream outputStream;
  if ( !message.empty() ) outputStream << message << endl;
  outputStream << createMessage( type, toCatch );
  message = outputStream.str();
}

std::string xml_errorhandler::createMessage
  ( const std::string &type, const SAXParseException &toCatch )
{
  ostringstream outputStream;
  outputStream << type.c_str() << " at file \""
               << xml_strx<char,XMLCh>(toCatch.getSystemId()).getBuffer()
               << "\", line " << toCatch.getLineNumber()
               << ", column " << toCatch.getColumnNumber()
               << ": "
               << xml_strx<char,XMLCh>(toCatch.getMessage()).getBuffer();
  return outputStream.str();
}

}

#endif
