#include "mimasconfig.h"

#ifdef HAVE_XALAN

#ifdef SIMPLEXPATH
#include <XPath/XPathEvaluator.hpp>
#include <DOMSupport/XalanDocumentPrefixResolver.hpp>
#else
#include <XPath/XPath.hpp>
#include <XPath/ElementPrefixResolverProxy.hpp>
#include <XPath/XPathProcessorImpl.hpp>
#endif
#include "xml_document.h"
#include "xml_reference.h"
#include "xml_node_reference_list.h"

using namespace boost;

namespace mimas {

template <>
xml_reference< XalanNode > xml_reference< XalanElement >::selectNode
  ( const char *xPath ) const throw (mimasexception,XMLException)
{
  xml_node_reference_list nodeList( selectNodes( xPath ) );
  MMERROR( nodeList.getLength() >= 1, mimasexception, ,
           "Could not find XML node with x-path \"" << xPath  << "\"." );
  MMERROR( nodeList.getLength() <= 1, mimasexception, ,
           "There are multiple nodes with x-path \"" << xPath << "\"." );
  assert( nodeList.getLength() == 1 );
  return nodeList.item( 0 );
}

template <>
xml_node_reference_list xml_reference< XalanElement >::selectNodes
  ( const char *xPath ) const throw (mimasexception,XMLException)
{
  MMERROR( document != NULL, mimasexception, ,
           "Can not select nodes from empty XML document." );
#ifdef SIMPLEXPATH
  XalanDocumentPrefixResolver thePrefixResolver( document );
  XPathEvaluator theEvaluator;
  NodeRefList obj =
    theEvaluator.selectNodeList( *document->getDOMSupport(), node,
                                 XalanDOMString( xPath ).c_str(),
                                 thePrefixResolver );
#else
  XPathProcessorImpl theXPathProcessor;
  XPath * const contextXPath = document->getXPathFactory()->create();
  ElementPrefixResolverProxy prefixResolver
    ( node, *document->getEnvSupport(),
      *document->getDOMSupport() );
  theXPathProcessor.initXPath( *contextXPath,
  // With which version was this introduced exactly?

#if _XALAN_VERSION >= 10500
                               *document->getXPathConstructionContext(),
#endif
                               XalanDOMString( xPath ),
                               prefixResolver );
  shared_ptr< XObjectPtr > obj( new XObjectPtr );
  *obj =
    contextXPath->execute( node,
                           prefixResolver,
                           *document->getExecutionContext() );
#endif
  return xml_node_reference_list( obj, document );
}

template <>
bool xml_reference< XalanElement >::existNode( const char *xPath ) const
  throw (XMLException)
{
  return selectNodes( xPath ).getLength() >= 1;
}

};

#endif
